<?php
/**
 * @package     CleverReach
 * @subpackage  cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\BusinessLogic\Entity\Tag;
use CleverReach\BusinessLogic\Entity\TagCollection;
use CleverReach\BusinessLogic\Sync\RecipientSyncTask;
use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\Infrastructure\ServiceRegister;
use CleverReach\BusinessLogic\Sync\RecipientDeactivateSyncTask;
use CleverReach\BusinessLogic\Sync\FilterSyncTask;
use CleverReach\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\Infrastructure\TaskExecution\Queue;
use CleverReach\Infrastructure\TaskExecution\QueueItem;
use Joomla\Utilities\ArrayHelper;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class PlgUserCleverReach extends JPlugin
{
	const NEWSLETTER_FIELD_NAME = 'cleverreach_newsletter_subscription';

	/**
	 * @var array
	 * @since version
	 */
	private static $oldTags = array();

	/**
	 * @var array Stores user IDs of all users who have tags
	 * of groups that are deleted.
	 *
	 * @since 1.0.0
	 */
	private static $usersWithDeletedTags = array();

	/**
	 * @var bool Signifies if user group details have been changed.
	 *
	 * @since 1.0.0
	 */
	private static $groupModified = false;

	/**
	 * Retrieves information about user's newsletter subscription status if it exists in the database.
	 *
	 * @param   string $context The context for the data.
	 * @param   object $data    User data.
	 *
	 * @return    boolean
	 *
	 * @since 1.0.0
	 */
	public function onContentPrepareData($context, $data)
	{
		if (is_array($data)
			|| !in_array($context, array(
					'com_users.profile',
					'com_users.registration',
					'com_users.user',
					'com_admin.profile'
				)
			)
		)
		{
			return true;
		}

		if (isset($data->id))
		{
			$userId = $data->id;
		}
		else
		{
			$userId = 0;
		}

		$userRepository      = new CleverreachRepositoryUser();
		$result              = $userRepository->getUserNewsletterStatus($userId);
		$newsletterFieldName = self::NEWSLETTER_FIELD_NAME;

		if (!isset($data->newsletter))
		{
			$data->newsletter = new \stdClass();
		}

		if ($result !== null)
		{
			$data->newsletter->$newsletterFieldName = $result[1];
		}
		else
		{
			$data->newsletter->$newsletterFieldName = 0;
		}

		return true;
	}

	/**
	 * Hooks on form rendering to add an additional field for newsletter subscription.
	 *
	 * @param   JForm $form The form to be altered.
	 *
	 * @return boolean
	 *
	 * @since 1.0.0
	 */
	public function onContentPrepareForm(JForm $form)
	{
		$lang = JFactory::getLanguage();
		$lang->load('cleverreach', JPATH_ADMINISTRATOR);

		if (!($form instanceof JForm))
		{
			$this->_subject->setError('JERROR_NOT_A_FORM');

			return false;
		}

		if (!in_array($form->getName(), array(
				'com_users.profile',
				'com_users.registration',
				'com_users.user',
				'com_admin.profile'
			)
		)
		)
		{
			return true;
		}

		JForm::addFormPath(__DIR__ . '/fields');
		$form->loadFile(self::NEWSLETTER_FIELD_NAME);
		$form->setFieldAttribute(self::NEWSLETTER_FIELD_NAME, 'required', true, 'account');

		return true;
	}

	/**
	 * Hooks before saving user data to check if user has been modified and to resync it to CleverReach if necessary.
	 *
	 * @param   array $oldUser User data before update.
	 * @param   bool  $isNew   Boolean to identify if this is a new user or an existing one.
	 * @param   array $newUser User data after update.
	 *
	 * @return boolean
	 *
	 * @throws QueueStorageUnavailableException
	 *
	 * @since 1.0.0
	 */
	public function onUserBeforeSave($oldUser, $isNew, $newUser)
	{
		if ($this->handleEvent())
		{
			$this->handleSiteNameChanged();

			if (!$isNew && $this->mailOrGroupsModified($oldUser, $newUser))
			{
				if ($oldUser['email'] !== $newUser['email'])
				{
					CleverreachUtilityTaskqueue::enqueue(new RecipientDeactivateSyncTask(array($oldUser['email'])));
				}

				$userRepository = new CleverreachRepositoryUser;
				$deletedGroups  = array();
				if (is_array($oldUser['groups']) && is_array($newUser['groups']))
				{
					$deletedGroups  = array_keys(array_diff($oldUser['groups'], $newUser['groups']));
				}

				self::$oldTags  = new TagCollection;

				if (!empty($deletedGroups))
				{
					self::$oldTags = $this->formatTags($userRepository->getGroups($deletedGroups));
				}
			}
		}

		return true;
	}

	/**
	 * Hooks on saving user data and adds newsletter subscription field in user profiles table
	 *
	 * @param   array $data   An associative array of the columns in the user table.
	 * @param   bool  $isNew  Boolean to identify if this is a new user or an existing one.
	 * @param   bool  $result Boolean to identify if the store was successful.
	 *
	 * @return boolean
	 *
	 * @throws QueueStorageUnavailableException
	 *
	 * @since 1.0.0
	 */
	public function onUserAfterSave($data, $isNew, $result)
	{
		$userId = ArrayHelper::getValue($data, 'id', 0, 'int');

		$this->storeNewsletterSubscriptionStatus($data, $result, $userId);

		if ($isNew)
		{
			$this->syncNewRecipient($userId);
		}
		else
		{
			CleverreachUtilityTaskqueue::enqueue(new RecipientSyncTask(array($userId), self::$oldTags));
		}

		return true;
	}

	/**
	 * Hooks on this event to deactivate user on CleverReach.
	 *
	 * @param   array $user An associative array of the columns in the user table.
	 *
	 * @return boolean
	 *
	 * @throws QueueStorageUnavailableException
	 *
	 * @since 1.0.0
	 */
	public function onUserBeforeDelete($user)
	{
		if ($this->handleEvent())
		{
			$this->handleSiteNameChanged();
			CleverreachUtilityTaskqueue::enqueue(new RecipientDeactivateSyncTask(array($user['email'])));
		}

		return true;
	}

	/**
	 * Remove newsletter subscription information for the given user ID.
	 *
	 * @param   array   $user    Holds the user data
	 * @param   boolean $success True if user was successfully stored in the database
	 *
	 * @return boolean
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function onUserAfterDelete($user, $success)
	{
		if (!$success)
		{
			return false;
		}

		$userId = ArrayHelper::getValue($user, 'id', 0, 'int');

		if ($userId)
		{
			$userRepository = new CleverreachRepositoryUser();
			$userRepository->deleteUserSubscriptionStatus($userId);
		}

		return true;
	}

	/**
	 * Hooks before user group details have been saved to store old tags
	 * if there has been a change to group title or hierarchy
	 * in order to remove them from CleverReach after the group details
	 * have been saved to the database
	 *
	 * @param   string   $context The context for the data.
	 * @param   stdClass $oldData Group data before save event.
	 * @param   bool     $isNew   Boolean to identify if this is a new group or an existing one.
	 * @param   array    $newData Group data after save event.
	 *
	 * @return boolean
	 *
	 * @throws QueueStorageUnavailableException
	 *
	 * @since 1.0.0
	 */
	public function onUserBeforeSaveGroup($context, $oldData, $isNew, $newData)
	{
		if ($this->handleEvent())
		{
			$this->handleSiteNameChanged();

			if ($isNew)
			{
				CleverreachUtilityTaskqueue::enqueue(new FilterSyncTask);
			}
			else
			{
				$userRepository = new CleverreachRepositoryUser;

				// Both $oldData and $newData hold the same values, so we need to fetch old group information
				// from the database
				$oldGroupData = $userRepository->getGroup($oldData->id);

				if ($oldGroupData->title !== $newData['title'] || $oldGroupData->parent_id !== $newData['parent_id'])
				{
					self::$groupModified = true;

					// Save old tags into a static variable
					$subGroupIds   = $userRepository->getSubgroups($oldData->id, array());
					$groupIds      = array_merge(array($oldData->id), $subGroupIds);
					self::$oldTags = $userRepository->getGroups($groupIds);
				}
			}
		}

		return true;
	}

	/**
	 * Hooks after user group details have been saved to resync
	 * all recipients within this group and all of its subgroups
	 * if there has been a change to group title or hierarchy
	 * because the hierarchy has been updated in the database
	 *
	 * @param   string   $context The context for the data.
	 * @param   stdClass $oldData Group data before save event.
	 * @param   bool     $isNew   Boolean to identify if this is a new group or an existing one.
	 * @param   array    $newData Group data after save event.
	 *
	 * @return boolean
	 *
	 * @throws QueueStorageUnavailableException
	 *
	 * @since 1.0.0
	 */
	public function onUserAfterSaveGroup($context, $oldData, $isNew, $newData)
	{
		if (!$isNew && self::$groupModified)
		{
			$userRepository = new CleverreachRepositoryUser;

			CleverreachUtilityTaskqueue::enqueue(new FilterSyncTask);

			$groupName   = $userRepository->getGroups(array($oldData->id));
			$userIds     = $userRepository->getUsersByGroup($oldData->id);
			$subGroupIds = $userRepository->getSubgroups($oldData->id, array());
			$userIds     = $this->mergeUsersFromSubgroups($userIds, $subGroupIds, $userRepository);

			if (empty($groupName))
			{
				$emails = $userRepository->getUserEmails($userIds);
				CleverreachUtilityTaskqueue::enqueue(new RecipientDeactivateSyncTask($emails));
			}
			else
			{
				CleverreachUtilityTaskqueue::enqueue(new RecipientSyncTask($userIds, $this->formatTags(self::$oldTags)));
			}

			self::$groupModified = false;
		}

		return true;
	}

	/**
	 * Hooks before group has been deleted to store all old tags in order to remove them from CR later
	 *
	 * @param   array $group Group data.
	 *
	 * @return boolean
	 *
	 * @throws QueueStorageUnavailableException
	 *
	 * @since 1.0.0
	 */
	public function onUserBeforeDeleteGroup($group)
	{
		if ($this->handleEvent())
		{
			$this->handleSiteNameChanged();
			$userRepository             = new CleverreachRepositoryUser;
			self::$usersWithDeletedTags = $userRepository->getUsersByGroup($group['id']);

			// Save old tags into a static variable
			$subGroupIds                = $userRepository->getSubgroups((int) $group['id'], array());
			$groupIds                   = array_merge(array($group['id']), $subGroupIds);
			self::$oldTags              = $userRepository->getGroups($groupIds);
			self::$usersWithDeletedTags = $this->mergeUsersFromSubgroups(
				self::$usersWithDeletedTags,
				$subGroupIds,
				$userRepository
			);
		}

		return true;
	}

	/**
	 * Hooks after the group has been deleted to resync all recipients and remove old tags
	 *
	 * @return boolean
	 *
	 * @throws QueueStorageUnavailableException
	 *
	 * @since 1.0.0
	 */
	public function onUserAfterDeleteGroup()
	{
		if ($this->handleEvent())
		{
			CleverreachUtilityTaskqueue::enqueue(new FilterSyncTask);
			CleverreachUtilityTaskqueue::enqueue(
				new RecipientSyncTask(self::$usersWithDeletedTags, $this->formatTags(self::$oldTags))
			);
		}

		return true;
	}

	/**
	 * Sends new user to CleverReach sync.
	 *
	 * @param   int $userId ID of the user.
	 *
	 * @return void
	 *
	 * @throws QueueStorageUnavailableException
	 *
	 * @since 1.0.0
	 */
	public function syncNewRecipient($userId)
	{
		if ($this->handleEvent() && !$this->handleSiteNameChanged())
		{
			CleverreachUtilityTaskqueue::enqueue(new RecipientSyncTask(array($userId)));
		}
	}

	/**
	 * Saves newsletter subscription status (true or false) to user profiles table.
	 *
	 * @param   array $data   User data.
	 * @param   bool  $result Boolean to identify if the store was successful
	 * @param   int   $userId ID of the user.
	 *
	 * @return void
	 *
	 * @since version
	 */
	public function storeNewsletterSubscriptionStatus($data, $result, $userId)
	{
		if ($userId && $result && isset($data['newsletter']))
		{
			$userRepository = new CleverreachRepositoryUser();
			$userRepository->setUserNewsletterStatus($userId, $data['newsletter'][self::NEWSLETTER_FIELD_NAME]);
		}
	}

	/**
	 * Function that checks if auth process and initial sync has been finished,
	 * as well as if the required core and loader classes exist in current Joomla! installation
	 * and allows the incoming event to be handled
	 *
	 * @return boolean
	 *
	 * @since 1.0.0
	 */
	private function handleEvent()
	{
		$accessToken         = $this->getConfigService()->getAccessToken();
		$initialSyncTaskItem = $this->getQueueService()->findLatestByType('CleverreachServiceSyncCustominitialsynctask');

		if ($initialSyncTaskItem === null)
		{
			return false;
		}

		return $accessToken !== null
			&& in_array($initialSyncTaskItem->getStatus(), array(QueueItem::IN_PROGRESS, QueueItem::COMPLETED), true)
			&& $this->requiredClassesExist();
	}

	/**
	 * Checks if required core and loader classes exist in current Joomla! installation.
	 *
	 * @return boolean
	 *   If classes exist returns true, otherwise false.
	 *
	 * @since 1.0.0
	 */
	private function requiredClassesExist()
	{
		$core          = JLibraryHelper::getLibrary('cleverreach_core', true);
		$loaderEnabled = JPluginHelper::isEnabled('system', 'cleverreach_loader');

		if ($core === null)
		{
			return false;
		}

		return $core->enabled && $loaderEnabled;
	}

	/**
	 * Check if user's email or groups have been modified.
	 *
	 * @param   array $oldUser User data before updating
	 * @param   array $newUser User data after updating
	 *
	 * @return boolean
	 *
	 * @since 1.0.0
	 */
	private function mailOrGroupsModified($oldUser, $newUser)
	{
		return $oldUser['email'] !== $newUser['email'] || !$this->arraysEqual($oldUser['groups'], $newUser['groups']);
	}

	/**
	 * Checks if the site name has changes and, if so, enqueues all users for resync.
	 *
	 * @return boolean Return value signifies that the site name has been changed
	 *              and that all recipients have been resynced.
	 *
	 * @throws \CleverReach\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException
	 *
	 * @since 1.0.0
	 */
	private function handleSiteNameChanged()
	{
		$config = new JConfig;

		if ($config->sitename !== $this->getConfigService()->getSiteName())
		{
			$this->getConfigService()->setSiteName($config->sitename);
			$userRepository = new CleverreachRepositoryUser;
			$userIds        = $userRepository->getAllUserIds();
			CleverreachUtilityTaskqueue::enqueue(new RecipientSyncTask($userIds));

			return true;
		}

		return false;
	}

	/**
	 * Creates a tag collection with provided tags in that collection.
	 *
	 * @param   array $tags Array of strings that need to be formatted as tags.
	 *
	 * @return TagCollection
	 *
	 * @since 1.0.0
	 */
	private function formatTags(array $tags)
	{
		$tagCollection = new TagCollection;

		foreach ($tags as $tag)
		{
			$tagCollection->addTag(new Tag($tag, CleverreachServiceBusinesslogicRecipients::GROUP_TYPE));
		}

		return $tagCollection;
	}

	/**
	 * Returns configuration service.
	 *
	 * @return CleverreachServiceInfrastructureConfig
	 *
	 * @since 1.0.0
	 */
	private function getConfigService()
	{
		return ServiceRegister::getService(Configuration::CLASS_NAME);
	}

	/**
	 * Returns queue service.
	 *
	 * @return Queue
	 *
	 * @since 1.0.0
	 */
	private function getQueueService()
	{
		return ServiceRegister::getService(Queue::CLASS_NAME);
	}

	/**
	 * Checks if two arrays are identical (but with possible shuffle between array elements).
	 *
	 * @param   array $a First array.
	 * @param   array $b Second array.
	 *
	 * @return boolean
	 *
	 * @since 1.0.0
	 */
	private function arraysEqual($a, $b)
	{
		return (
			is_array($a)
			&& is_array($b)
			&& count($a) === count($b)
			&& array_diff($a, $b) === array_diff($b, $a)
		);
	}

	/**
	 * Merges users from base group with users from all subgroups.
	 *
	 * @param   array                     $userIds        Array of user IDs.
	 * @param   array                     $subGroupIds    Array of all subgroups of a parent group.
	 * @param   CleverreachRepositoryUser $userRepository User repository.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	private function mergeUsersFromSubgroups($userIds, $subGroupIds, $userRepository)
	{
		if (empty($subGroupIds))
		{
			return $userIds;
		}

		$subGroupUserIds = array($userIds);

		foreach ($subGroupIds as $subGroupId)
		{
			$subGroupUserIds[] = $userRepository->getUsersByGroup((int) $subGroupId);
		}

		return call_user_func_array('array_merge', $subGroupUserIds);
	}
}
