<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\Infrastructure\ServiceRegister;
use CleverReach\Infrastructure\TaskExecution\Queue;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachControllerBase extends JControllerLegacy
{
	/**
	 * @var CleverreachServiceInfrastructureConfig
	 *
	 * @since 1.0.0
	 */
	protected $configService;

	/**
	 * @var Queue
	 *
	 * @since 1.0.0
	 */
	protected $queue;

	/**
	 * Processes request. Reads 'action' parameter and calls action method if provided.
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function process()
	{
		$input = JFactory::getApplication()->input;

		$action = $this->getParam('action', $input);

		if ($action)
		{
			if (method_exists($this, $action))
			{
				$this->$action();
			}

			$this->returnJson(array('error' => "Method $action does not exist!"));
		}
	}

	/**
	 * Calls task queue wakeup.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public function wakeUp()
	{
		CleverreachUtilityTaskqueue::wakeup();
	}

	/**
	 * Sets response header content type to json, echos supplied $data as a json string and terminates request.
	 *
	 * @param   array $data Array to be returned as a json response
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	protected function returnJson(array $data)
	{
		$app = JFactory::getApplication();

		$app->mimeType = 'application/json';
		$app->setHeader('Content-Type', $app->mimeType . '; charset=' . $app->charSet);
		$app->setHeader('status', 200, true);
		$app->sendHeaders();
		echo json_encode($data);

		$app->close();
	}

	/**
	 * Checks whether current request is POST.
	 *
	 * @return boolean
	 *
	 * @since 1.0.0
	 */
	protected function isPost()
	{
		return $_SERVER['REQUEST_METHOD'] === 'POST';
	}

	/**
	 * Gets request parameter if exists. Otherwise, returns null.
	 *
	 * @param   string  $key   Key of the parameter.
	 * @param   \JInput $input The application input object.
	 *
	 * @return mixed
	 *
	 * @since 1.0.0
	 */
	protected function getParam($key, $input)
	{
		$param = $input->get($key);

		return !empty($param) ? $param : null;
	}

	/**
	 * Gets raw request.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	protected function getRawInput()
	{
		return file_get_contents('php://input');
	}

	/**
	 * Returns 404 response and terminates request.
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	protected function redirect404()
	{
		$app = JFactory::getApplication();

		$app->setHeader('status', 404, true);
		$app->sendHeaders();

		$app->close();
	}

	/**
	 * Check if the caller of the internal controller method is logged in
	 *
	 * @return boolean
	 *
	 * @since 1.0.0
	 */
	protected function isLoggedIn()
	{
		$user = JFactory::getUser();

		return $user->id !== 0;
	}

	/**
	 * Returns configuration service.
	 *
	 * @return CleverreachServiceInfrastructureConfig
	 *
	 * @since 1.0.0
	 */
	protected function getConfigService()
	{
		if ($this->configService === null)
		{
			$this->configService = ServiceRegister::getService(Configuration::CLASS_NAME);
		}

		return $this->configService;
	}

	/**
	 * Returns queue service.
	 *
	 * @return Queue
	 *
	 * @since 1.0.0
	 */
	protected function getQueueService()
	{
		if ($this->queue === null)
		{
			$this->queue = ServiceRegister::getService(Queue::CLASS_NAME);
		}

		return $this->queue;
	}

}
