<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\BusinessLogic\Sync\RefreshUserInfoTask;
use CleverReach\Infrastructure\ServiceRegister;
use CleverReach\BusinessLogic\Interfaces\Proxy;
use CleverReach\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\Infrastructure\TaskExecution\Queue;
use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use Joomla\CMS\Language\Text;
use CleverReach\Infrastructure\Utility\Exceptions\HttpCommunicationException;
use CleverReach\Infrastructure\TaskExecution\Exceptions\TaskRunnerStatusStorageUnavailableException;
use CleverReach\BusinessLogic\Entity\AuthInfo;
use CleverReach\Infrastructure\Exceptions\BadAuthInfoException;
use Joomla\Registry\Registry;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachControllerCallback extends CleverreachControllerBase
{
	/**
	 *
	 * @throws HttpCommunicationException
	 * @throws TaskRunnerStatusStorageUnavailableException
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function process()
	{
		parent::process();

		$input             = JFactory::getApplication()->input;

		$this->validateRequest($input);

		/** @var CleverreachServiceInfrastructureConfig $config */
		$config            = ServiceRegister::getService(Configuration::CLASS_NAME);
		$config->setTaskRunnerStatus('', null);
		$component         = JComponentHelper::getComponent('com_cleverreach', true);
		$extension         = \JTable::getInstance('extension');
		$extension->load($component->id);
		$manifest          = new Registry($extension->manifest_cache);
		$version           = $manifest->get('version');
		$config->setLastUpdateVersion($version);

		if ($input->get('refresh'))
		{
			$authInfo = $this->getAuthInfo($input, true);
			$this->refreshTokens($authInfo);
		}
		else
		{
			$authInfo = $this->getAuthInfo($input, false);
			$this->queueRefreshUserTask($authInfo);
		}

		echo '<script>window.close()</script>';
	}

	/**
	 * Refreshes user access tokens.
	 *
	 * @param   AuthInfo $authInfo User auth info object.
	 *
	 * @return void
	 *
	 * @throws HttpCommunicationException
	 * @throws \CleverReach\Infrastructure\Exceptions\InvalidConfigurationException
	 * @throws \CleverReach\Infrastructure\Utility\Exceptions\HttpRequestException
	 * @throws \CleverReach\Infrastructure\Utility\Exceptions\RefreshTokenExpiredException
	 *
	 * @since 1.0.0
	 */
	private function refreshTokens($authInfo)
	{
		/** @var CleverreachServiceInfrastructureConfig $config */
		$config = ServiceRegister::getService(Configuration::CLASS_NAME);
		/** @var \CleverReach\BusinessLogic\Proxy $proxy */
		$proxy = ServiceRegister::getService(Proxy::CLASS_NAME);

		$userInfo = $proxy->getUserInfo($authInfo->getAccessToken());
		$localInfo = $config->getUserInfo();

		if (isset($userInfo['id']) && $userInfo['id'] === $localInfo['id'])
		{
			$config->setAccessToken($authInfo->getAccessToken());
			$config->setRefreshToken($authInfo->getRefreshToken());
			$config->setAccessTokenExpirationTime($authInfo->getAccessTokenDuration());
		}
	}

	/**
	 * Validates HTTP request
	 *
	 * @param   JInput $input Application input object.
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	private function validateRequest($input)
	{
		$code = $this->getParam('code', $input);

		if (empty($code))
		{
			$this->returnJson(array(
					'status'  => false,
					'message' => Text::_('COM_CLEVERREACH_WRONG_PARAMS'),
				)
			);
		}
	}

	/**
	 * Retrieves auth info.
	 *
	 * @param   JInput $input         Application input object.
	 * @param   bool   $refreshTokens Whether this call is made to refresh tokens or not.
	 *
	 * @return mixed
	 *
	 * @throws HttpCommunicationException
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	private function getAuthInfo($input, $refreshTokens)
	{
		$code = $this->getParam('code', $input);

		/** @var \CleverReach\BusinessLogic\Proxy $proxy */
		$proxy       = ServiceRegister::getService(Proxy::CLASS_NAME);
		$redirectUrl = $refreshTokens ? CleverreachUtilityHelper::getRefreshAuthRedirectUrl()
			: CleverreachUtilityHelper::getAuthRedirectUrl();
		try
		{
			/** @var AuthInfo $result */
			$result = $proxy->getAuthInfo($code, $redirectUrl);
		}
		catch (BadAuthInfoException $e)
		{
			$this->returnJson(
				array(
					'status'  => false,
					'message' => $e->getMessage() ?: Text::_('COM_CLEVERREACH_UNSUCCESSFUL_CONNECTION'),
				)
			);
		}

		return $result;
	}

	/**
	 * Enqueues refresh user info task for initial sync.
	 *
	 * @param   AuthInfo $authInfo CleverReach auth info object.
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	private function queueRefreshUserTask($authInfo)
	{
		/** @var Queue $queue */
		$queue         = ServiceRegister::getService(Queue::CLASS_NAME);
		$configService = ServiceRegister::getService(Configuration::CLASS_NAME);

		try
		{
			$queue->enqueue($configService->getQueueName(), new RefreshUserInfoTask($authInfo));
		}
		catch (QueueStorageUnavailableException $e)
		{
			$this->returnJson(
				array(
					'status'  => false,
					'message' => $e->getMessage(),
				)
			);
		}
	}
}
