<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Logger\Logger;
use CleverReach\Infrastructure\TaskExecution\Exceptions\QueueItemDeserializationException;
use CleverReach\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\Infrastructure\TaskExecution\QueueItem;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachControllerCheckstatus extends CleverreachControllerBase
{
	/**
	 * Gets status of refreshing user info task.
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function userInfo()
	{
		$status = 'finished';

		/** @var QueueItem $queueItem */
		$queueItem = $this->getQueueService()->findLatestByType('RefreshUserInfoTask');

		if ($queueItem !== null)
		{
			$queueStatus = $queueItem->getStatus();

			if ($queueStatus !== QueueItem::FAILED && $queueStatus !== QueueItem::COMPLETED)
			{
				$status = 'in_progress';
			}
			elseif ($queueStatus === QueueItem::COMPLETED)
			{
				$config = new JConfig;
				$this->getConfigService()->setSiteName($config->sitename);
				$lang = JFactory::getLanguage();
				$this->getConfigService()->setUserLanguage($lang->getTag());
			}
		}

		$this->returnJson(array('status' => $status));
	}

	/**
	 * Gets status of initial sync task.
	 *
	 * @throws QueueItemDeserializationException
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function initialSync()
	{
		/** @var QueueItem $syncTaskQueueItem */
		$syncTaskQueueItem = $this->getQueueService()->findLatestByType('CleverreachServiceSyncCustominitialsynctask');

		if ($syncTaskQueueItem === null)
		{
			$this->returnJson(array('status' => QueueItem::FAILED));
		}

		/** @var CleverreachServiceSyncCustominitialsynctask $initialSyncTask */
		$initialSyncTask         = $syncTaskQueueItem->getTask();
		$initialSyncTaskProgress = $initialSyncTask->getProgressByTask();

		$this->returnJson(
			array(
				'status'       => $syncTaskQueueItem->getStatus(),
				'taskStatuses' => array(
					'subscriber_list' => array(
						'status'   => $this->getStatus($initialSyncTaskProgress['subscriberList']),
						'progress' => $initialSyncTaskProgress['subscriberList'],
					),
					'add_fields'      => array(
						'status'   => $this->getStatus($initialSyncTaskProgress['fields']),
						'progress' => $initialSyncTaskProgress['fields'],
					),
					'recipient_sync'  => array(
						'status'   => $this->getStatus($initialSyncTaskProgress['recipients']),
						'progress' => $initialSyncTaskProgress['recipients'],
					),
				),
			)
		);
	}

	/**
	 * Enqueues initial sync task.
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function retrySync()
	{
		try
		{
			CleverreachUtilityTaskqueue::enqueue(new CleverreachServiceSyncCustominitialsynctask, true);
		}
		catch (QueueStorageUnavailableException $e)
		{
			Logger::logError('Error restarting sync: ' . $e->getMessage());
			$this->returnJson(array('status' => 'failed'));
		}

		$this->returnJson(array('status' => 'success'));
	}

	/**
	 * Marks that the first email is built.
	 *
	 * @throws \InvalidArgumentException
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function buildFirstEmail()
	{
		$this->getConfigService()->setIsFirstEmailBuilt('1');

		$this->returnJson(array('status' => 'success'));
	}

	/**
	 * Based on progress, gives the status of a queued item
	 *
	 * @param   int $progress Integer value between 0 and 100, representing task progress.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	private function getStatus($progress)
	{
		$status = QueueItem::QUEUED;

		if (0 < $progress && $progress < 100)
		{
			$status = QueueItem::IN_PROGRESS;
		}
		elseif ($progress >= 100)
		{
			$status = QueueItem::COMPLETED;
		}

		return $status;
	}
}
