<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Interfaces\Required\AsyncProcessStarter;
use CleverReach\Infrastructure\ServiceRegister;
use Joomla\Input\Json;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachControllerConfig extends CleverreachControllerBase
{
	/**
	 * Processes request. Reads 'action' parameter and calls action method if provided.
	 *
	 * @return void
	 *
	 * @throws QueueItemDeserializationException
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function process()
	{
		$app = JFactory::getApplication();

		if ($this->isPost())
		{
			$result = $this->update($app->input);
		}
		else
		{
			$result = $this->getConfigParameters();
		}

		$this->returnJson($result);
	}

	/**
	 * Returns all configuration parameters for diagnostics purposes.
	 *
	 * @return array Configuration values.
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	private function getConfigParameters()
	{
		/** @var CleverreachServiceInfrastructureAsyncprocessstarter $asyncProcessService */
		$asyncProcessService = ServiceRegister::getService(AsyncProcessStarter::CLASS_NAME);

		return array(
			'integrationId'                      => $this->getConfigService()->getIntegrationId(),
			'integrationName'                    => $this->getConfigService()->getIntegrationName(),
			'minLogLevel'                        => $this->getConfigService()->getMinLogLevel(),
			'isProductSearchEnabled'             => $this->getConfigService()->isProductSearchEnabled(),
			'productSearchParameters'            => $this->getConfigService()->getProductSearchParameters(),
			'recipientsSynchronizationBatchSize' => $this->getConfigService()->getRecipientsSynchronizationBatchSize(),
			'isDefaultLoggerEnabled'             => $this->getConfigService()->isDefaultLoggerEnabled(),
			'maxStartedTasksLimit'               => $this->getConfigService()->getMaxStartedTasksLimit(),
			'maxTaskExecutionRetries'            => $this->getConfigService()->getMaxTaskExecutionRetries(),
			'maxTaskInactivityPeriod'            => $this->getConfigService()->getMaxTaskInactivityPeriod(),
			'taskRunnerMaxAliveTime'             => $this->getConfigService()->getTaskRunnerMaxAliveTime(),
			'taskRunnerStatus'                   => $this->getConfigService()->getTaskRunnerStatus(),
			'taskRunnerWakeupDelay'              => $this->getConfigService()->getTaskRunnerWakeupDelay(),
			'queueName'                          => $this->getConfigService()->getQueueName(),
			'logLocation'                        => CleverreachServiceInfrastructureLogger::getLogLocation(),
			'joomlaVersion'                      => JVERSION,
			'asyncProcessStarterUrl'             => $asyncProcessService->getAsyncProcessStarterUrl(),
			'eventHandlerUrl'                    => $this->getConfigService()->getCrEventHandlerURL(),
		);
	}

	/**
	 * Updates configuration from POST request.
	 *
	 * @param   JInput $input Application input object.
	 *
	 * @return array Success message.
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	private function update($input)
	{
		/** @var Json $json */
		$json = $input->json;
		$data = json_decode($json->getRaw());

		if (isset($data->minLogLevel))
		{
			$this->getConfigService()->saveMinLogLevel($data->minLogLevel);
		}

		if (isset($data->defaultLoggerStatus))
		{
			$this->getConfigService()->setDefaultLoggerEnabled($data->defaultLoggerStatus);
		}

		if (isset($data->maxStartedTasksLimit))
		{
			$this->getConfigService()->setMaxStartedTaskLimit($data->maxStartedTasksLimit);
		}

		if (isset($data->taskRunnerWakeUpDelay))
		{
			$this->getConfigService()->setTaskRunnerWakeUpDelay($data->taskRunnerWakeUpDelay);
		}

		if (isset($data->taskRunnerMaxAliveTime))
		{
			$this->getConfigService()->setTaskRunnerMaxAliveTime($data->taskRunnerMaxAliveTime);
		}

		if (isset($data->maxTaskExecutionRetries))
		{
			$this->getConfigService()->setMaxTaskExecutionRetries($data->maxTaskExecutionRetries);
		}

		if (isset($data->maxTaskInactivityPeriod))
		{
			$this->getConfigService()->setMaxTaskInactivityPeriod($data->maxTaskInactivityPeriod);
		}

		if (isset($data->productSearchEndpointPassword))
		{
			$this->getConfigService()->setProductSearchEndpointPassword($data->productSearchEndpointPassword);
		}

		if (isset($data->configuredInitialSync))
		{
			$this->getConfigService()->setConfiguredInitialSync($data->configuredInitialSync);
		}

		if (isset($data->defaultRecipientStatus))
		{
			$this->getConfigService()->setDefaultRecipientStatus(!empty($data->defaultRecipientStatus));
		}

		return array('status' => 'success', 'message' => 'Successfully updated config values!');
	}

}
