<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\BusinessLogic\Utility\ArticleSearch\Conditions;
use CleverReach\BusinessLogic\Utility\ArticleSearch\Schema\Enum;
use CleverReach\BusinessLogic\Utility\ArticleSearch\Schema\EnumSchemaAttribute;
use CleverReach\BusinessLogic\Utility\ArticleSearch\Schema\SimpleCollectionSchemaAttribute;
use CleverReach\BusinessLogic\Utility\ArticleSearch\Schema\SimpleSchemaAttribute;
use CleverReach\BusinessLogic\Utility\ArticleSearch\Schema\SchemaAttributeTypes;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchResult\TextAttribute;
use CleverReach\Infrastructure\ServiceRegister;
use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use Joomla\CMS\Language\Language;
use Joomla\DI\Exception\KeyNotFoundException;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachRepositoryArticle extends CleverreachRepositoryBase
{
	const TABLE_CONTENT = '#__content';
	const TABLE_TAGS = '#__tags';
	const TABLE_CONTENT_TAG_MAP = '#__contentitem_tag_map';
	const TABLE_CATEGORIES = '#__categories';

	/**
	 * Array of mappings for status names and values representing them in content table.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	private $statusMappings = array(
		'trashed'     => -2,
		'unpublished' => 0,
		'published'   => 1,
		'archived'    => 2,
	);

	/**
	 * CleverreachRepositoryArticle constructor.
	 *
	 * @since       1.0.0
	 */
	public function __construct()
	{
		parent::__construct();

		$this->tableName = self::TABLE_CONTENT;
		$this->idColumn  = 'id';
	}

	/**
	 * Returns article as the only supported searchable item in Joomla!
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	public function getSearchableItems()
	{
		return array(
			'article' => 'Article',
		);
	}

	/**
	 * Always returns article, since that is the only searchable item supported on Joomla!
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public function getContentType()
	{
		return 'article';
	}

	/**
	 * Returns article schema with all of searchable attributes.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	public function getStaticSchema()
	{
		$language       = $this->getTranslationLanguage();
		$knownLanguages = JLanguage::getKnownLanguages();
		$languages      = array();
		$languages[]    = new Enum('Default', '*');
		$statuses       = array();

		foreach ($knownLanguages as $knownLanguage)
		{
			$languages[] = new Enum($knownLanguage['name'], $knownLanguage['tag']);
		}

		foreach ($this->statusMappings as $key => $value)
		{
			$statuses[] = new Enum($key, $value);
		}

		return array(
			new SimpleSchemaAttribute(
				'title',
				$language->_('JGLOBAL_TITLE'),
				true,
				array(
					Conditions::EQUALS,
					Conditions::CONTAINS,
				),
				SchemaAttributeTypes::TEXT
			),
			new SimpleSchemaAttribute(
				'alias',
				$language->_('JFIELD_ALIAS_LABEL'),
				true,
				array(
					Conditions::EQUALS,
					Conditions::CONTAINS,
				),
				SchemaAttributeTypes::TEXT
			),
			new EnumSchemaAttribute(
				'status',
				$language->_('JSTATUS'),
				true,
				array(
					Conditions::EQUALS,
					Conditions::NOT_EQUAL,
				),
				$statuses
			),
			new SimpleSchemaAttribute(
				'content',
				$language->_('COM_CONTENT_ARTICLE_CONTENT'),
				true,
				array(
					Conditions::CONTAINS,
				),
				SchemaAttributeTypes::HTML
			),
			new SimpleSchemaAttribute(
				'featured',
				$language->_('COM_CONTENT_FEATURED'),
				true,
				array(
					Conditions::EQUALS,
				),
				'bool'
			),
			new SimpleSchemaAttribute(
				'category',
				$language->_('JCATEGORY'),
				true,
				array(
					Conditions::EQUALS,
					Conditions::CONTAINS,
				),
				SchemaAttributeTypes::TEXT
			),
			new SimpleCollectionSchemaAttribute(
				'tags',
				$language->_('JTAG'),
				true,
				array(
					Conditions::CONTAINS,
				),
				'text'
			),
			new SimpleSchemaAttribute(
				'intro_image',
				$language->_('COM_CONTENT_FIELD_INTRO_LABEL'),
				false,
				array(),
				SchemaAttributeTypes::IMAGE
			),
			new SimpleSchemaAttribute(
				'article_image',
				$language->_('COM_CONTENT_FIELD_FULL_LABEL'),
				false,
				array(),
				SchemaAttributeTypes::IMAGE
			),
			new SimpleSchemaAttribute(
				'created_date',
				$language->_('COM_CONTENT_FIELD_CREATED_LABEL'),
				true,
				array(
					Conditions::EQUALS,
					Conditions::GREATER_THAN,
					Conditions::GREATER_EQUAL,
					Conditions::LESS_THAN,
					Conditions::LESS_EQUAL,
				),
				SchemaAttributeTypes::DATE
			),
			new SimpleSchemaAttribute(
				'start_publishing_date',
				$language->_('MOD_POPULAR_OPTION_STARTPUBLISHING_VALUE'),
				true,
				array(
					Conditions::EQUALS,
					Conditions::GREATER_THAN,
					Conditions::GREATER_EQUAL,
					Conditions::LESS_THAN,
					Conditions::LESS_EQUAL,
				),
				SchemaAttributeTypes::DATE
			),
			new SimpleSchemaAttribute(
				'end_publishing_date',
				$language->_('COM_CONTACT_FIELD_PUBLISH_DOWN_LABEL'),
				true,
				array(
					Conditions::EQUALS,
					Conditions::GREATER_THAN,
					Conditions::GREATER_EQUAL,
					Conditions::LESS_THAN,
					Conditions::LESS_EQUAL,
				),
				SchemaAttributeTypes::DATE
			),
			new SimpleSchemaAttribute(
				'author',
				$language->_('JAUTHOR'),
				true,
				array(
					Conditions::EQUALS,
					Conditions::CONTAINS,
				),
				SchemaAttributeTypes::AUTHOR
			),
			new SimpleSchemaAttribute(
				'author_alias',
				$language->_('COM_BANNERS_FIELD_CREATED_BY_ALIAS_LABEL'),
				true,
				array(
					Conditions::EQUALS,
					Conditions::CONTAINS,
				),
				SchemaAttributeTypes::TEXT
			),
			new SimpleSchemaAttribute(
				'url',
				'URL',
				false,
				array(),
				SchemaAttributeTypes::URL
			),
			new EnumSchemaAttribute(
				'language',
				$language->_('MOD_LOGIN_LANGUAGE'),
				true,
				array(
					Conditions::EQUALS,
					Conditions::NOT_EQUAL,
				),
				$languages
			)
		);
	}

	/**
	 * Returns article(s) that meet the requirements of provided filter(s).
	 *
	 * @param   array $filters Array of filters with field name, value and search condition.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	public function getFilteredArticles($filters)
	{
		$subQuery = $this->db->getQuery(true);
		$query    = $this->db->getQuery(true);

		$subQuery->select(array(
				'a.id AS id',
				'a.title AS title',
				'a.alias AS alias',
				'a.state AS status',
				'a.introtext',
				'a.fulltext',
				'a.created AS created_date',
				'a.created_by_alias AS author_alias',
				'a.publish_up AS start_publishing_date',
				'a.publish_down AS end_publishing_date',
				'a.images AS images',
				'a.featured AS featured',
				'a.language AS language',
				'c.title AS category',
				'u.name AS author'
			)
		)
			->from($this->db->quoteName($this->tableName) . ' AS a')
			->innerJoin($this->db->quoteName(CleverreachRepositoryUser::TABLE_USER) . ' AS u ON a.created_by=u.id')
			->innerJoin($this->db->quoteName(self::TABLE_CATEGORIES) . ' AS c ON a.catid = c.id');

		$query->select($this->buildSelect(array()))
			->from($subQuery, 'selected_columns');

		if ($this->buildCondition($filters) !== '')
		{
			$query->where($this->buildCondition($filters));
		}

		$tagFilter = $this->getTagFilter($filters);

		if (!empty($tagFilter) && $tagFilter['value'] !== '')
		{
			return $this->getArticlesByTags($query, $tagFilter);
		}

		$this->db->setQuery($query);

		return $this->db->loadObjectList();
	}

	/**
	 * Returns all tags for the article with the provided ID.
	 *
	 * @param   int $id Article ID.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	public function getArticleTags($id)
	{
		$tagHelper = new JHelperTags;
		$tags      = $tagHelper->getItemTags('com_content.article', $id);

		$tagNames = array();

		foreach ($tags as $tag)
		{
			$tagNames[] = new TextAttribute('title', $tag->title);
		}

		return $tagNames;
	}

	/**
	 * Returns status name [published, unpublished, archived, trashed] for a given status value.
	 *
	 * @param   int $value Integer value for article status.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public function getStatusName($value)
	{
		$name = array_search((int) $value, $this->statusMappings, true);

		if ($name === false)
		{
			throw new KeyNotFoundException('Value for a non-existing article status');
		}

		return $name;
	}

	/**
	 * Builds where condition as part of select query
	 *
	 * @param   array $filterBy Conditions for select query (key-value pairs).
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	protected function buildCondition($filterBy)
	{
		if (empty($filterBy))
		{
			return '';
		}

		$where = array();

		foreach ($filterBy as $filter)
		{
			if ($filter['field'] === 'content')
			{
				$where[] = '(' . $this->db->quoteName('introtext') . ' LIKE \'%' . $filter['value'] . '%\''
					. ' OR ' . $this->db->quoteName('fulltext') . ' LIKE \'%' . $filter['value'] . '%\')';
			}
			elseif ($filter['field'] !== 'itemCode' && $filter['field'] !== 'tags')
			{
				switch ($filter['condition'])
				{
					case Conditions::EQUALS:
						$where[] = $this->db->quoteName($filter['field'])
							. ' = ' . $this->db->quote($this->getConditionValue($filter));
						break;
					case Conditions::NOT_EQUAL:
						$where[] = $this->db->quoteName($filter['field'])
							. ' <> ' . $this->db->quote($this->getConditionValue($filter));
						break;
					case Conditions::CONTAINS:
						$where[] = $this->db->quoteName($filter['field'])
							. " LIKE '%" . $this->getConditionValue($filter) . "%'";
						break;
					case Conditions::GREATER_THAN:
						$where[] = $this->db->quoteName($filter['field'])
							. ' > ' . $this->db->quote($this->getConditionValue($filter));
						break;
					case Conditions::GREATER_EQUAL:
						$where[] = $this->db->quoteName($filter['field'])
							. ' >= ' . $this->db->quote($this->getConditionValue($filter));
						break;
					case Conditions::LESS_THAN:
						$where[] = $this->db->quoteName($filter['field'])
							. ' < ' . $this->db->quote($this->getConditionValue($filter));
						break;
					case Conditions::LESS_EQUAL:
						$where[] = $this->db->quoteName($filter['field'])
							. ' <= ' . $this->db->quote($this->getConditionValue($filter));
						break;
					default:
						$where[] = '';
				}
			}
		}

		return implode(' AND ', $where);
	}

	/**
	 * Takes already formed select query and adds filter for required tags.
	 *
	 * @param   JDatabaseQuery $query     Existing query formed in caller method.
	 * @param   array          $tagFilter Array of preformatted tag filters.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	private function getArticlesByTags($query, $tagFilter)
	{
		$tagNames = explode(',', $tagFilter['value']);

		$articlesQuery = $this->db->getQuery(true);
		$articlesQuery->select(array(
				'a.id',
				'COUNT(a.id)',
				'a.title',
				'a.alias',
				'a.status',
				'a.introtext',
				'a.fulltext',
				'a.created_date',
				'a.author_alias',
				'a.start_publishing_date',
				'a.end_publishing_date',
				'a.images',
				'a.featured',
				'a.language',
				'a.category',
				'a.author'
			)
		)
			->from($query, 'a')
			->innerJoin($this->db->quoteName(self::TABLE_CONTENT_TAG_MAP) . ' AS m ON m.content_item_id=a.id')
			->innerJoin($this->db->quoteName(self::TABLE_TAGS) . ' AS t ON m.tag_id=t.id')
			->where(array('t.title IN (\'' . implode('\',\'', $tagNames) . '\')'))
			->group('a.id')
			->having('COUNT(a.id) = ' . count($tagNames));

		$this->db->setQuery($articlesQuery);

		return $this->db->loadObjectList();
	}

	/**
	 * Returns filter for tags if exists.
	 *
	 * @param   array $filters Array of parsed and verified filters.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	private function getTagFilter($filters)
	{
		$tagFilter = array();

		foreach ($filters as $filter)
		{
			if ($filter['field'] === 'tags')
			{
				$tagFilter = $filter;
			}
		}

		return $tagFilter;
	}

	/**
	 * Checks if filter is for status attribute, and returns status value in that case.
	 *
	 * @param   array $filter Array of preformatted seach filters.
	 *
	 * @return mixed
	 *
	 * @since 1.0.0
	 */
	private function getConditionValue($filter)
	{
		if ($filter['field'] === 'status')
		{
			return $this->getStatusValue($filter['value']);
		}

		return $this->escape($filter['value']);
	}


	/**
	 * Returns integer value for provided status name.
	 *
	 * @param   string $name Value from the following set [published, unpublished, archived, trashed].
	 *
	 * @return integer
	 *
	 * @since 1.0.0
	 */
	private function getStatusValue($name)
	{
		if (!isset($this->statusMappings[$name]))
		{
			throw new KeyNotFoundException('Article status doesn\'t exist');
		}

		return $this->statusMappings[$name];
	}

	/**
	 * Gets language that labels should be translated to.
	 *
	 * @return Language
	 *
	 * @since 1.0.0
	 */
	private function getTranslationLanguage()
	{
		$language = JFactory::getLanguage();
		/** @var string $tag Language tag (ex. en-GB) */
		$tag    = $this->getConfigService()->getUserLanguage();
		$locale = explode('-', $tag);
		$tag    = in_array($locale[0], array('en', 'de', 'it', 'fr', 'es')) ? $tag : 'en-GB';
		$language->load('com_content', JPATH_ADMINISTRATOR, $tag);
		$language->load('com_contact', JPATH_ADMINISTRATOR, $tag);
		$language->load('com_banners', JPATH_ADMINISTRATOR, $tag);
		$language->load('com_cleverreach', JPATH_ADMINISTRATOR, $tag);
		$language->load('mod_articles_popular', JPATH_BASE, $tag);
		$language->load('mod_login', JPATH_ADMINISTRATOR, $tag);

		return $language;
	}

	/**
	 * Returns configuration service.
	 *
	 * @return CleverreachServiceInfrastructureConfig
	 *
	 * @since 1.0.0
	 */
	private function getConfigService()
	{
		return ServiceRegister::getService(Configuration::CLASS_NAME);
	}
}
