<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachRepositoryBase
{
	const QUERY_LIMIT = 100;

	/**
	 * @var JDatabaseDriver $db A database connector object.
	 *
	 * @since 1.0.0
	 */
	protected $db;

	/**
	 * @var string
	 *
	 * @since 1.0.0
	 */
	protected $tableName;

	/**
	 * @var string
	 *
	 * @since 1.0.0
	 */
	protected $idColumn = 'id';

	/**
	 * BaseRepository constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct()
	{
		$this->db = JFactory::getDbo();
	}

	/**
	 * Inserts single record and returns insert ID, if available.
	 *
	 * @param   array $record Array of key-value pairs representing columns and values of a single record.
	 *
	 * @return integer
	 *
	 * @since 1.0.0
	 */
	public function insert($record)
	{
		$query = $this->db->getQuery(true);

		$query->insert($this->db->quoteName($this->tableName))
			->columns($this->db->quoteName(array_keys($record)))
			->values(implode(',', $this->escapeValues($record)));

		$this->db->setQuery($query);
		$this->db->execute();

		return (int) $this->db->insertid();
	}

	/**
	 * Updates single record.
	 *
	 * @param   array $record     An item of columns and values of a single record as key-value pairs.
	 * @param   array $conditions List of simple search filters as key-value pairs to find records to update.
	 *
	 * @return mixed
	 *
	 * @since 1.0.0
	 */
	public function update($record, $conditions)
	{
		$query = $this->db->getQuery(true);

		if (empty($conditions) && empty($records[$this->idColumn]))
		{
			throw new \InvalidArgumentException('Condition needs to be set for update');
		}

		$itemSet = array();

		foreach ($record as $field => $value)
		{
			$itemSet[] = $this->db->quoteName($field) . ' = ' . $this->escapeValue($value);
		}

		$conditionSet = array();

		foreach ($conditions as $field => $value)
		{
			if ($value === null)
			{
				$conditionSet[] = $this->db->quoteName($field) . ' IS NULL';
			}
			else
			{
				$conditionSet[] = $this->db->quoteName($field) . ' = ' . $this->escapeValue($value);
			}
		}

		$query->update($this->tableName)
			->set($itemSet)
			->where($conditionSet);

		$this->db->setQuery($query);

		return $this->db->execute();
	}

	/**
	 * Deletes a record.
	 *
	 * @param   array $conditions Conditions for WHERE clause.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public function delete($conditions)
	{
		$query = $this->db->getQuery(true);

		if (empty($conditions))
		{
			throw new \InvalidArgumentException('Condition needs to be set for update');
		}

		$query->delete()
			->from($this->db->quoteName($this->tableName))
			->where($this->buildCondition($conditions));

		$this->db->setQuery($query);
		$this->db->execute();
	}

	/**
	 * Finds record by primary key.
	 *
	 * @param   int $id Record ID.
	 *
	 * @return stdClass|null
	 *
	 * @since 1.0.0
	 */
	public function findByPk($id)
	{
		return $this->findOne(array($this->idColumn => $id));
	}

	/**
	 * Finds one record by provided conditions.
	 *
	 * @param   array $filterBy List of simple search filters as key-value pair. Leave empty for unfiltered result.
	 * @param   array $sortBy   List of sorting options where key is field and value is sort direction
	 *                          ("ASC" or "DESC"). Leave empty for default sorting.
	 *
	 * @return stdClass|null
	 *
	 * @since 1.0.0
	 */
	public function findOne($filterBy = null, $sortBy = null)
	{
		return $this->findAll($filterBy, $sortBy, null, 0, 1);
	}

	/**
	 * Finds all records for provided conditions ordered in provided sort.
	 *
	 * @param   array $filterBy List of simple search filters as key-value pair. Leave empty for unfiltered result.
	 * @param   array $sortBy   List of sorting options where key is field and value is sort direction
	 *                          ("ASC" or "DESC"). Leave empty for default sorting.
	 * @param   array $select   List of table columns to return. Column names could have alias as well.
	 *                          If empty, all columns are returned
	 * @param   int   $offset   From which record index result set should start.
	 * @param   int   $limit    Max number of records that should be returned
	 *
	 * @return array|stdClass List of stdClass objects or one object.
	 *
	 * @since 1.0.0
	 */
	public function findAll($filterBy = null, $sortBy = null, $select = null, $offset = -1, $limit = self::QUERY_LIMIT)
	{
		/** @var JDatabaseQuery $query */
		$query = $this->db->getQuery(true);

		$query->select($this->buildSelect($select))
			->from($this->db->quoteName($this->tableName));

		if (!empty($filterBy))
		{
			$query->where($this->buildCondition($filterBy));
		}

		if (!empty($sortBy))
		{
			$query->order($this->buildOrderBy($sortBy));
		}

		if ($offset !== -1)
		{
			$this->db->setQuery($query, $offset, $limit);
		}
		else
		{
			$this->db->setQuery($query);
		}

		if ($limit === 1)
		{
			return $this->db->loadObject();
		}

		return $this->db->loadObjectList();
	}

	/**
	 * Performs single DB escape of a value.
	 *
	 * @param   string $value Value to escape.
	 *
	 * @return mixed
	 *
	 * @since 1.0.0
	 */
	protected function escape($value)
	{
		return $this->db->escape($value);
	}

	/**
	 * Escapes value and adds database quotes around it. If value is NULL, returns NULL string.
	 *
	 * @param   mixed $value Value to escape and quote.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	protected function escapeValue($value)
	{
		return $value === null ? 'NULL' : $this->db->quote($value, true);
	}

	/**
	 * Escapes given values and prepares them for query.
	 * Values will already have needed quotes so do not apply quotes around values in your query.
	 *
	 * @param   array $values Array of values to escape.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	protected function escapeValues($values)
	{
		$result = array();

		foreach ($values as $value)
		{
			$result[] = $this->escapeValue($value);
		}

		return $result;
	}

	/**
	 * Builds select part of select query.
	 *
	 * @param   array $select Columns that are being requested.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	protected function buildSelect($select)
	{
		if (empty($select))
		{
			return array('*');
		}

		$result = array();

		foreach ($select as $field)
		{
			$result[] = $this->db->quoteName($field);
		}

		return $result;
	}

	/**
	 * Builds where condition as part of select query.
	 *
	 * @param   array $filterBy Conditions for select query (key-value pairs).
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	protected function buildCondition($filterBy)
	{
		if (empty($filterBy))
		{
			return '';
		}

		$where = array();

		foreach ($filterBy as $key => $value)
		{
			if ($value === null)
			{
				$where[] = $this->db->quoteName($key) . ' IS NULL';
			}
			elseif ($value === 'not null')
			{
				$where[] = "$key IS NOT NULL";
			}
			elseif (is_array($value) && !empty($value))
			{
				$where[] = $this->db->quoteName($key) . ' IN (' . implode(',', $this->escapeValues($value)) . ')';
			}
			else
			{
				$where[] = $this->db->quoteName($key) . ' = ' . $this->escapeValue($value);
			}
		}

		return implode(' AND ', $where);
	}

	/**
	 * Builds order by condition as part of select query.
	 *
	 * @param   array $orderBy Key-value pairs representing column and sort order (ASC or DESC)
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	protected function buildOrderBy($orderBy)
	{
		if (empty($orderBy))
		{
			return '';
		}

		$sort = array();

		foreach ($orderBy as $key => $order)
		{
			$sort[] = $this->db->quoteName($key) . ' ' . $order;
		}

		return implode(', ', $sort);
	}
}
