<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Interfaces\Required\TaskQueueStorage;
use CleverReach\Infrastructure\TaskExecution\QueueItem;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachRepositoryQueue extends CleverreachRepositoryBase
{
	/**
	 * QueueRepository constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct()
	{
		parent::__construct();

		$this->tableName = CleverreachTableQueue::QUEUE_TABLE;
	}

	/**
	 * Finds latest queue item by type.
	 *
	 * @param   string $type Type of queue item.
	 *
	 * @return array|null
	 *
	 * @since 1.0.0
	 */
	public function findLatest($type)
	{
		return $this->findOne(array('type' => $type), array('queueTimestamp' => TaskQueueStorage::SORT_DESC));
	}

	/**
	 * Finds list of earliest queued queue items per queue. Following list of criteria for searching must be satisfied:
	 *      - Queue must be without already running queue items
	 *      - For one queue only one (oldest queued) item should be returned
	 *
	 * @param   int $limit Result set limit. By default max 10 earliest queue items will be returned.
	 *
	 * @return array Found queue item list.
	 *
	 * @since 1.0.0
	 */
	public function findOldestQueuedItems($limit = 10)
	{
		$runningQueuesQuery = $this->db->getQuery(true);

		$runningQueuesQuery->select($this->buildSelect(array('queueName')))
			->from($this->db->quoteName($this->tableName))
			->where($this->buildCondition(array('status' => QueueItem::IN_PROGRESS)));

		$query = $this->db->getQuery(true);

		$query->select($this->buildSelect(array()))
			->from($this->db->quoteName($this->tableName))
			->where(array(
				$this->db->quoteName('status') . ' = \'' . QueueItem::QUEUED . '\'',
				$this->db->quoteName('queueName') . ' NOT IN (' . $runningQueuesQuery . ')'
				)
			)
			->group($this->db->quoteName('queueName'))
			->order($this->buildOrderBy(array('queueTimestamp' => 'ASC')));

		$this->db->setQuery($query, 0, $limit);
		$result = $this->db->loadObjectList();

		return !empty($result) ? $result : null;
	}
}
