<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

defined('_JEXEC') or die;

/**
 * Class CleverreachRepositoryUser
 *
 * @since       1.0.0
 */
class CleverreachRepositoryUser extends CleverreachRepositoryBase
{
	const TABLE_USER = '#__users';
	const TABLE_USER_PROFILE = '#__user_profiles';
	const TABLE_USER_GROUPS = '#__usergroups';
	const TABLE_USER_GROUP_MAP = '#__user_usergroup_map';
	const NEWSLETTER_FIELD_NAME = 'cleverreach_newsletter_subscription';

	/**
	 * CleverreachRepositoryUser constructor.
	 *
	 * @since       1.0.0
	 */
	public function __construct()
	{
		parent::__construct();

		$this->tableName = self::TABLE_USER;
		$this->idColumn  = 'id';
	}

	/**
	 * Gets IDs of all users.
	 *
	 * @return array
	 *
	 * @since       1.0.0
	 */
	public function getAllUserIds()
	{
		$ids = $this->findAll(array('NULLIF(email, \'\')' => 'not null'), null, array('id'));

		if (!empty($ids))
		{
			$ids = array_map(
				function ($userId) {
					return $userId->id;
				},
				$ids
			);

			return $ids;
		}

		return array();
	}

	/**
	 * Return email addresses for users with provided IDs.
	 *
	 * @param   array $userIds Array of user IDs.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	public function getUserEmails($userIds)
	{
		$query = $this->db->getQuery(true);

		$query->select($this->buildSelect(array('email')))
			->from($this->db->quoteName(self::TABLE_USER))
			->where($this->buildCondition(array('id' => $userIds)));

		$this->db->setQuery($query);

		$users = $this->db->loadObjectList();

		if ($users !== null)
		{
			return array_map(
				function ($user) {
					return $user->email;
				},
				$users
			);
		}

		return array();
	}

	/**
	 * Gets customers with provided IDs. Response has the following keys:
	 *   id, email, registered, first_name, last_name, language, street,
	 *   zip, city, state, country, birthday, phone, newsletter, groups
	 *
	 * @param   array $userIds Array of user IDs.
	 *
	 * @return array
	 *
	 * @since       1.0.0
	 */
	public function getUsers(array $userIds = array())
	{
		$response       = array();
		$users          = $this->getUsersWithBaseAttributes($userIds);
		$profileEnabled = JPluginHelper::isEnabled('user', 'profile');
		$profile        = array();
		$street         = '';

		foreach ($users as $user)
		{
			$userProfile = JUserHelper::getProfile($user->id);

			if ($profileEnabled)
			{
				$profile = $userProfile->profile;
				$street  = (!empty($profile['address1']) ? $profile['address1'] : '')
					. (!empty($profile['address2']) ? ' ' . $profile['address2'] : '');
			}

			$newsletterField        = self::NEWSLETTER_FIELD_NAME;
			$newsletterSubscription = !empty($userProfile->newsletter->$newsletterField)
				? $userProfile->newsletter->$newsletterField
				: false;
			$params                 = json_decode($user->params);

			// String will be divided into first name containing first part of full name
			// and last name containing the rest of the full name
			$fullName            = explode(' ', $user->name, 2);
			$response[$user->id] = array(
				'id'         => $user->id,
				'email'      => $user->email,
				'registered' => $user->registerDate,
				'first_name' => $fullName[0],
				'last_name'  => (count($fullName) === 2) ? $fullName[1] : '',
				'language'   => !empty($params->language) ? $params->language : '',
				'street'     => $street,
				'zip'        => isset($profile['postal_code']) ? $profile['postal_code'] : '',
				'city'       => isset($profile['city']) ? $profile['city'] : '',
				'state'      => isset($profile['region']) ? $profile['region'] : '',
				'country'    => isset($profile['country']) ? $profile['country'] : '',
				'birthday'   => isset($profile['dob']) ? $profile['dob'] : '',
				'phone'      => isset($profile['phone']) ? $profile['phone'] : '',
				'newsletter' => $newsletterSubscription ? true : false,
				'groups'     => $this->getUserGroups($user->id),
			);
		}

		return $response;
	}

	/**
	 * Returns IDs from users that belong to a group with provided ID.
	 *
	 * @param   int $groupId ID of the group.
	 *
	 * @return mixed
	 *
	 * @since       1.0.0
	 */
	public function getUsersByGroup($groupId)
	{
		$query = $this->db->getQuery(true);

		$query->select($this->buildSelect(array('u.id')))
			->from($this->db->quoteName(self::TABLE_USER) . ' AS u')
			->innerJoin($this->db->quoteName(self::TABLE_USER_GROUP_MAP) . ' AS g ON u.id = g.user_id')
			->where($this->buildCondition(array('g.group_id' => $groupId)));

		$this->db->setQuery($query);

		$users = $this->db->loadObjectList();

		if ($users !== null)
		{
			return array_map(
				function ($user) {
					return $user->id;
				},
				$users
			);
		}

		return array();
	}

	/**
	 * Retrieves user with a given e-mail.
	 *
	 * @param string $email
	 *
	 * @return stdClass|null
	 *
	 * @since 1.0.0
	 */
	public function getUserByMail($email)
	{
		return $this->findOne(array('email' => $email));
	}

	/**
	 * Returns all subgroups of a group with a given ID.<
	 *
	 * @param   int   $groupId   ID of the group.
	 * @param   array $subGroups Array of subgroups of a given group.
	 *
	 * @return array
	 *
	 * @since       1.0.0
	 */
	public function getSubgroups($groupId, $subGroups)
	{
		$query = $this->db->getQuery(true);

		$query->select($this->buildSelect(array('id', 'parent_id')))
			->from(self::TABLE_USER_GROUPS)
			->where($this->buildCondition(array('parent_id' => $groupId)));

		$this->db->setQuery($query);

		$groups = $this->db->loadObjectList();

		if ($groups !== null)
		{
			$groupIds = array_map(
				function ($group) {
					return $group->id;
				},
				$groups
			);

			$subGroups = array_merge($subGroups, $groupIds);

			foreach ($groupIds as $subGroup)
			{
				$subGroups = $this->getSubgroups((int) $subGroup, $subGroups);
			}
		}

		return $subGroups;
	}

	/**
	 * Returns full names (tags) of all requested groups.
	 *
	 * @param   array $groupIds Array of group IDs.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	public function getGroups(array $groupIds = array())
	{
		$query = $this->db->getQuery(true);
		$query->select($this->buildSelect(array('id', 'parent_id', 'title')))
			->from($this->db->quoteName(self::TABLE_USER_GROUPS));

		if (!empty($groupIds))
		{
			$query->where($this->buildCondition(array('id' => $groupIds)));
		}

		$this->db->setQuery($query);
		$groups     = $this->db->loadObjectList();
		$groupNames = array();

		foreach ($groups as $group)
		{
			$groupNames[] = $this->getFullGroupName($group);
		}

		return $groupNames;
	}

	/**
	 * Returns group details for a group with a given ID.
	 *
	 * @param   int $groupId ID of the group.
	 *
	 * @return stdClass
	 *
	 * @since 1.0.0
	 */
	public function getGroup($groupId)
	{
		$query = $this->db->getQuery(true);
		$query->select($this->buildSelect(array('id', 'parent_id', 'title')))
			->from($this->db->quoteName(self::TABLE_USER_GROUPS))
			->where($this->buildCondition(array('id' => $groupId)));
		$this->db->setQuery($query);

		return $this->db->loadObject();
	}

	/**
	 * Returns newsletter subscription status for a given user.
	 *
	 * @param   int $userId ID of the user.
	 *
	 * @return mixed
	 *
	 * @since 1.0.0
	 */
	public function getUserNewsletterStatus($userId)
	{
		$query = $this->db->getQuery(true);
		$query->select($this->buildSelect(array('profile_key', 'profile_value')))
			->from($this->db->quoteName(self::TABLE_USER_PROFILE))
			->where($this->buildCondition(
				array(
					'user_id'     => $userId,
					'profile_key' => self::NEWSLETTER_FIELD_NAME
				)
			)
			);

		$this->db->setQuery($query);

		return $this->db->loadRow();
	}

	/**
	 * Saves user newsletter status.
	 *
	 * @param   int $userId           ID of the user.
	 * @param   int $newsletterStatus Newsletter status.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public function setUserNewsletterStatus($userId, $newsletterStatus)
	{
		if ($this->getUserNewsletterStatus($userId) !== null)
		{
			$this->deleteUserSubscriptionStatus($userId);
		}

		$query  = $this->db->getQuery(true);
		$values = array(
			$this->db->quote($userId),
			$this->db->quote(self::NEWSLETTER_FIELD_NAME),
			$this->db->quote($newsletterStatus)
		);

		$query->insert($this->db->quoteName(self::TABLE_USER_PROFILE))
			->columns($this->db->quoteName(array('user_id', 'profile_key', 'profile_value')))
			->values(implode(',', $values));

		$this->db->setQuery($query);

		if (!$this->db->execute())
		{
			throw new RuntimeException('Database execution failed');
		}
	}

	/**
	 * Deletes user newsletter status.
	 *
	 * @param   int $userId ID of the user.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public function deleteUserSubscriptionStatus($userId)
	{
		$query = $this->db->getQuery(true);
		$query->delete($this->db->quoteName(self::TABLE_USER_PROFILE))
			->where($this->buildCondition(
				array(
					'user_id'     => $userId,
					'profile_key' => self::NEWSLETTER_FIELD_NAME
				)
			)
			);

		$this->db->setQuery($query);

		if (!$this->db->execute())
		{
			throw new RuntimeException('Database execution failed');
		}
	}

	/**
	 * Sets all newsletter subscription status for users that do not have newsletter status defined.
	 *
	 * @param   bool $subscribeToNewsletter Boolean specifying whether users should be set as subsribers or not.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public function setDefaultNewsletterStatus($subscribeToNewsletter = false)
	{
		$userIds                     = $this->getAllUserIds();
		$existingNewsletterStatusIds = $this->getUsersWithNewsletterStatus();

		foreach ($userIds as $id)
		{
			if (!in_array($id, $existingNewsletterStatusIds, true))
			{
				$query = $this->db->getQuery(true);

				$query->insert($this->db->quoteName('#__user_profiles'))
					->columns($this->db->quoteName(array('user_id', 'profile_key', 'profile_value')))
					->values(
						implode(
							',',
							array(
								$this->db->quote($id),
								$this->db->quote('cleverreach_newsletter_subscription'),
								$this->db->quote($subscribeToNewsletter ? 1 : 0),
							)
						)
					);

				$this->db->setQuery($query);

				if (!$this->db->execute())
				{
					throw new RuntimeException('Database execution failed');
				}
			}
		}
	}

	/**
	 * Fetches all users from base table (without profile values like address, city, zip...).
	 *
	 * @param   array $userIds Array of user IDs.
	 *
	 * @return array
	 *
	 * @since       1.0.0
	 */
	private function getUsersWithBaseAttributes(array $userIds = array())
	{
		$condition = array();

		if (!empty($userIds))
		{
			$condition = array(
				'id' => $userIds,
			);
		}

		$users = $this->findAll(
			$condition,
			null,
			array(
				'id',
				'email',
				'registerDate',
				'name',
				'params',
			)
		);

		return $users;
	}

	/**
	 * Returns names of all groups that a given user belongs to.
	 *
	 * @param   int $id ID of the user.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	private function getUserGroups($id)
	{
		$query = $this->db->getQuery(true);

		$query->select($this->buildSelect(array('m.group_id', 'g.parent_id', 'g.title')))
			->from($this->db->quoteName(self::TABLE_USER_GROUP_MAP) . ' AS m')
			->innerJoin($this->db->quoteName(self::TABLE_USER_GROUPS) . ' AS g on g.id = m.group_id')
			->where($this->buildCondition(array('m.user_id' => $id)));

		$this->db->setQuery($query);
		$groups     = $this->db->loadObjectList();
		$groupNames = array();

		foreach ($groups as $group)
		{
			$name         = $this->getFullGroupName($group);
			$groupNames[] = $name;
		}

		return $groupNames;
	}

	/**
	 * Returns IDs of users who already have newsletter status defined.
	 *
	 * @return array Array of integers representing IDs of users who have newsletter status.
	 *
	 * @since 1.0.0
	 */
	private function getUsersWithNewsletterStatus()
	{
		$query = $this->db->getQuery(true);

		$query->select(array('*'))
			->from($this->db->quoteName('#__user_profiles'))
			->where(array($this->db->quoteName('profile_key')
					. ' = ' . $this->db->quote('cleverreach_newsletter_subscription'))
			);

		$this->db->setQuery($query);

		$usersWithNewsletterStatus = $this->db->loadObjectList();
		$userIds                   = array_map(
			function ($profileRecord) {
				return $profileRecord->user_id;
			},
			$usersWithNewsletterStatus
		);

		return $userIds;
	}

	/**
	 * Retrieves full group name, along with its hierarchy.
	 *
	 * @param   stdClass $group Group object.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	private function getFullGroupName($group)
	{
		if ((int) $group->parent_id !== 0)
		{
			$query = $this->db->getQuery(true);
			$query->select($this->buildSelect(array('parent_id', 'title')))
				->from(self::TABLE_USER_GROUPS)
				->where($this->buildCondition(array('id' => $group->parent_id)));
			$this->db->setQuery($query);

			$result = $this->db->loadObject();

			return $this->getFullGroupName($result) . '_' . $group->title;
		}

		return $group->title;
	}

}
