<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\BusinessLogic\Interfaces\Recipients;
use CleverReach\BusinessLogic\Entity\Recipient;
use CleverReach\Infrastructure\ServiceRegister;
use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\BusinessLogic\Entity\TagCollection;
use CleverReach\BusinessLogic\Entity\Tag;
use CleverReach\BusinessLogic\Entity\SpecialTagCollection;
use CleverReach\BusinessLogic\Entity\SpecialTag;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachServiceBusinesslogicRecipients implements Recipients
{
	const GROUP_TYPE = 'Group';

	/**
	 * @var CleverreachRepositoryUser
	 *
	 * @since 1.0.0
	 */
	private $userRepository;

	/**
	 * @var CleverreachServiceInfrastructureConfig
	 *
	 * @since 1.0.0
	 */
	private $configService;

	/**
	 * CleverreachServiceBusinesslogicRecipients constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct()
	{
		$this->userRepository = new CleverreachRepositoryUser;
	}

	/**
	 * Gets all tags as a collection.
	 *
	 * @return \CleverReach\BusinessLogic\Entity\TagCollection
	 *
	 * @since 1.0.0
	 */
	public function getAllTags()
	{
		return $this->formatTags($this->userRepository->getGroups());
	}

	/**
	 * Gets all special tags as a collection.
	 *
	 * @return \CleverReach\BusinessLogic\Entity\SpecialTagCollection
	 *
	 * @since 1.0.0
	 */
	public function getAllSpecialTags()
	{
		return new SpecialTagCollection(array(SpecialTag::contact(), SpecialTag::subscriber()));
	}

	/**
	 * Gets all recipients for passed batch id with tags formatted in the proper way.
	 *
	 * @param   array $batchRecipientIds Array of recipients IDs.
	 * @param   bool  $includeOrders     Whether or not to include orders in response.
	 *
	 * @return array of CleverReach\BusinessLogic\Entity\Recipient objects based on passed ids.
	 * - If includeOrders flag is set to true orders should also be returned with other recipient data. SPECIAL
	 * - ATTENTION should be pointed towards tags. They should be in instance specific formatted way. If instance has
	 * - groups G1 and G3 array of strings for tags should be: array(G-G1, G-G2,)
	 *
	 * @since 1.0.0
	 */
	public function getRecipientsWithTags(array $batchRecipientIds, $includeOrders)
	{
		$users               = $this->userRepository->getUsers($batchRecipientIds);
		$formattedRecipients = array();

		foreach ($users as $user)
		{
			$formattedRecipient = new Recipient($user['email']);
			$formattedRecipient->setRegistered(new DateTime($user['registered']));
			$formattedRecipient->setFirstName($user['first_name']);
			$formattedRecipient->setLastName($user['last_name']);
			$formattedRecipient->setBirthday(!empty($user['birthday']) ? new DateTime($user['birthday']) : null);
			$formattedRecipient->setSource(JUri::base());
			$formattedRecipient->setShop($this->getConfigService()->getSiteName());
			$formattedRecipient->setCustomerNumber($user['id']);
			$formattedRecipient->setLanguage($this->getLanguage($user['language']));
			$formattedRecipient->setStreet($user['street']);
			$formattedRecipient->setZip($user['zip']);
			$formattedRecipient->setCity($user['city']);
			$formattedRecipient->setState($user['state']);
			$formattedRecipient->setCountry($user['country']);
			$formattedRecipient->setPhone($user['phone']);
			$formattedRecipient->setNewsletterSubscription($user['newsletter']);
			$formattedRecipient->setActive($user['newsletter']);
			$formattedRecipient->setTags($this->formatTags($user['groups']));
			$formattedRecipient->setSpecialTags($this->formatSpecialTags($user['newsletter']));

			$formattedRecipients[] = $formattedRecipient;
		}

		return $formattedRecipients;
	}

	/**
	 * Gets all recipients IDs from source system.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	public function getAllRecipientsIds()
	{
		return $this->userRepository->getAllUserIds();
	}

	/**
	 * Informs service about completed synchronization of provided recipients (IDs).
	 *
	 * @param   array $recipientIds Array of recipient IDs.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public function recipientSyncCompleted(array $recipientIds)
	{
	}

	/**
	 * Returns a language in a format that CleverReach expects
	 *
	 * @param   string $language User language in format en-GB, de-DE etc.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public function getCleverreachLanguageFormat($language)
	{
		$languages    = JLanguage::getKnownLanguages();
		$userLanguage = $languages[$language];
		$languageName = explode(' ', strtolower($userLanguage['name']));

		return $languageName[0];
	}

	/**
	 * Creates a tag collection with provided tags in that collection.
	 *
	 * @param   array $tags Array of strings that need to be formatted as tags.
	 *
	 * @return TagCollection
	 *
	 * @since 1.0.0
	 */
	private function formatTags(array $tags)
	{
		$tagCollection = new TagCollection;

		foreach ($tags as $tag)
		{
			$tagCollection->addTag(new Tag($tag, self::GROUP_TYPE));
		}

		return $tagCollection;
	}

	/**
	 * Creates a special tag collection and returns it.
	 *
	 * @param   bool $newsletterSubscription Whether or not is given user subscribed to newsletter.
	 *
	 * @return SpecialTagCollection
	 *
	 * @since 1.0.0
	 */
	private function formatSpecialTags($newsletterSubscription)
	{
		$specialTags = new SpecialTagCollection;

		$specialTags->addTag(SpecialTag::contact());

		if ($newsletterSubscription)
		{
			$specialTags->addTag(SpecialTag::subscriber());
		}

		return $specialTags;
	}

	/**
	 * Returns language in a proper format.
	 *
	 * @param   string $language Language tag (ex. en-GB)
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	private function getLanguage($language)
	{
		if (empty($language))
		{
			$lang = JFactory::getLanguage();
			$languageName = explode(' ', strtolower($lang->getName()));

			return $languageName[0];
		}

		return $this->getCleverreachLanguageFormat($language);
	}

	/**
	 * Returns configuration service.
	 *
	 * @return CleverreachServiceInfrastructureConfig
	 *
	 * @since 1.0.0
	 */
	private function getConfigService()
	{
		if ($this->configService === null)
		{
			$this->configService = ServiceRegister::getService(Configuration::CLASS_NAME);
		}

		return $this->configService;
	}
}
