<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Interfaces\Required\AsyncProcessStarter;
use CleverReach\Infrastructure\Interfaces\Required\HttpClient;
use CleverReach\Infrastructure\Interfaces\Exposed\Runnable;
use CleverReach\Infrastructure\Logger\Logger;
use CleverReach\Infrastructure\ServiceRegister;
use CleverReach\Infrastructure\TaskExecution\Exceptions\ProcessStarterSaveException;
use CleverReach\Infrastructure\Utility\Exceptions\HttpRequestException;
use CleverReach\Infrastructure\Utility\GuidProvider;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachServiceInfrastructureAsyncprocessstarter implements AsyncProcessStarter
{
	/**
	 * @var HttpClient HTTP client service
	 *
	 * @since 1.0.0
	 */
	private $httpClientService;

	/**
	 * Starts given runner asynchronously (in new process/web request or similar).
	 *
	 * @param   Runnable $runner Runner that should be started asynchronously.
	 *
	 * @return void
	 *
	 * @throws ProcessStarterSaveException
	 * @throws HttpRequestException
	 *
	 * @since 1.0.0
	 */
	public function start(Runnable $runner)
	{
		$guidProvider = new GuidProvider;
		$guid         = trim($guidProvider->generateGuid());

		$this->saveGuidAndRunner($guid, $runner);
		$this->startRunnerAsynchronously($guid);
	}

	/**
	 * Gets URL for async process controller.
	 *
	 * @return string
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function getAsyncProcessStarterUrl()
	{
		return $this->formatUrl(0);
	}

	/**
	 * Saves runner and guid to storage.
	 *
	 * @param   string   $guid   ID of the process.
	 * @param   Runnable $runner Runnable object.
	 *
	 * @return void
	 *
	 * @throws ProcessStarterSaveException
	 *
	 * @since 1.0.0
	 */
	private function saveGuidAndRunner($guid, Runnable $runner)
	{
		try
		{
			$processRepository = new CleverreachRepositoryProcess;
			$processRepository->save($guid, $runner);
		}
		catch (\Exception $e)
		{
			Logger::logError($e->getMessage(), 'Integration');
			throw new ProcessStarterSaveException($e->getMessage(), 0, $e);
		}
	}

	/**
	 * Starts runnable asynchronously.
	 *
	 * @param   string $guid ID of the process.
	 *
	 * @return void
	 *
	 * @throws HttpRequestException
	 *
	 * @since 1.0.0
	 */
	private function startRunnerAsynchronously($guid)
	{
		try
		{
			$this->getHttpClient()->requestAsync('POST', $this->formatUrl($guid));
		}
		catch (\Exception $e)
		{
			Logger::logError($e->getMessage(), 'Integration');
			throw new HttpRequestException($e->getMessage(), 0, $e);
		}
	}

	/**
	 * Formats URL for async process controller.
	 *
	 * @param   string $guid ID of the process.
	 *
	 * @return string
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	private function formatUrl($guid)
	{
		return CleverreachUtilityHelper::getAsyncProcessStartUrl($guid);
	}

	/**
	 * Returns HTTP client
	 *
	 * @return HttpClient|mixed
	 *
	 * @since 1.0.0
	 */
	private function getHttpClient()
	{
		if ($this->httpClientService === null)
		{
			$this->httpClientService = ServiceRegister::getService(HttpClient::CLASS_NAME);
		}

		return $this->httpClientService;
	}
}
