<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\Infrastructure\Interfaces\Required\HttpClient;
use CleverReach\Infrastructure\ServiceRegister;
use CleverReach\Infrastructure\Utility\Exceptions\HttpCommunicationException;
use CleverReach\Infrastructure\Utility\HttpResponse;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachServiceInfrastructureHttpclient extends HttpClient
{
	/**
	 * @var resource cURL session handler.
	 * @since 1.0.0
	 */
	private $curlSession;

	/**
	 * Create and send request.
	 *
	 * @param   string $method  Request method.
	 * @param   string $url     URL for the request.
	 * @param   array  $headers Request headers.
	 * @param   string $body    Request body in JSON format.
	 *
	 * @return HttpResponse
	 *
	 * @throws HttpCommunicationException Only in situation when there is no connection, no response, throw this exception
	 *
	 * @since 1.0.0
	 */
	public function sendHttpRequest($method, $url, $headers = array(), $body = '')
	{
		$this->setCurlSessionAndCommonRequestParts($method, $url, $headers, $body);
		$this->setCurlSessionOptionsForSynchronousRequest();

		return $this->executeAndReturnResponseForSynchronousRequest($url);
	}

	/**
	 * Create and send request asynchronously.
	 *
	 * @param   string $method  Request method.
	 * @param   string $url     URL for the request.
	 * @param   array  $headers Request headers.
	 * @param   string $body    Request body in JSON format.
	 *
	 * @return mixed
	 *
	 * @since 1.0.0
	 */
	public function sendHttpRequestAsync($method, $url, $headers = array(), $body = '')
	{
		$this->setCurlSessionAndCommonRequestParts($method, $url, $headers, $body);
		$this->setCurlSessionOptionsForAsynchronousRequest();

		return curl_exec($this->curlSession);
	}

	/**
	 * Sets cURL session and common request parts.
	 *
	 * @param   string $method  Request method.
	 * @param   string $url     URL for the request.
	 * @param   array  $headers Request headers.
	 * @param   string $body    Request body in JSON format.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private function setCurlSessionAndCommonRequestParts($method, $url, array $headers, $body)
	{
		$this->initializeCurlSession();
		$this->setCurlSessionOptionsBasedOnMethod($method);
		$this->setCurlSessionUrlHeadersAndBody($url, $headers, $body);
		$this->setCommonOptionsForCurlSession();
	}

	/**
	 * Initializes cURL session.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private function initializeCurlSession()
	{
		$this->curlSession = curl_init();
	}

	/**
	 * Sets cURL session options.
	 *
	 * @param   string $method Request method.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private function setCurlSessionOptionsBasedOnMethod($method)
	{
		if ($method === 'DELETE')
		{
			curl_setopt($this->curlSession, CURLOPT_CUSTOMREQUEST, 'DELETE');
		}

		if ($method === 'POST')
		{
			curl_setopt($this->curlSession, CURLOPT_POST, true);
		}

		if ($method === 'PUT')
		{
			curl_setopt($this->curlSession, CURLOPT_CUSTOMREQUEST, 'PUT');
		}
	}

	/**
	 * Sets cURL session headers.
	 *
	 * @param   string $url     URL for the request.
	 * @param   array  $headers Request headers.
	 * @param   string $body    Request body in JSON format.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private function setCurlSessionUrlHeadersAndBody($url, array $headers, $body)
	{
		curl_setopt($this->curlSession, CURLOPT_URL, $url);
		curl_setopt($this->curlSession, CURLOPT_HTTPHEADER, $headers);

		if (!empty($body))
		{
			curl_setopt($this->curlSession, CURLOPT_POSTFIELDS, $body);
		}
	}

	/**
	 * Sets common options for cURL session.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private function setCommonOptionsForCurlSession()
	{
		curl_setopt($this->curlSession, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($this->curlSession, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($this->curlSession, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($this->curlSession, CURLOPT_SSL_VERIFYHOST, false);
	}

	/**
	 * Set cURL session options for synchronous header.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private function setCurlSessionOptionsForSynchronousRequest()
	{
		curl_setopt($this->curlSession, CURLOPT_HEADER, true);
	}

	/**
	 * Sets cURL session options for asynchronous request.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private function setCurlSessionOptionsForAsynchronousRequest()
	{
		// Always ensure the connection is fresh
		curl_setopt($this->curlSession, CURLOPT_FRESH_CONNECT, true);

		// Timeout super fast once connected, so it goes into async
		$timeout = ServiceRegister::getService(Configuration::CLASS_NAME)->getAsyncProcessRequestTimeout();
		curl_setopt($this->curlSession, CURLOPT_TIMEOUT_MS, $timeout);
	}

	/**
	 * Executes synchronous request and returns and response.
	 *
	 * @param   string $url Request URL.
	 *
	 * @return HttpResponse
	 * @throws HttpCommunicationException
	 *
	 * @since 1.0.0
	 */
	private function executeAndReturnResponseForSynchronousRequest($url)
	{
		$apiResponse = curl_exec($this->curlSession);
		$statusCode  = curl_getinfo($this->curlSession, CURLINFO_HTTP_CODE);
		curl_close($this->curlSession);

		if ($apiResponse === false)
		{
			throw new HttpCommunicationException('Request ' . $url . ' failed.');
		}

		return new HttpResponse(
			$statusCode,
			$this->getHeadersFromCurlResponse($apiResponse),
			$this->getBodyFromCurlResponse($apiResponse)
		);
	}

	/**
	 * Gets headers from cURL response.
	 *
	 * @param   string $response cURL response.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	private function getHeadersFromCurlResponse($response)
	{
		$headers              = array();
		$headersBodyDelimiter = "\r\n\r\n";
		$headerText           = substr($response, 0, strpos($response, $headersBodyDelimiter));
		$headersDelimiter     = "\r\n";

		foreach (explode($headersDelimiter, $headerText) as $i => $line)
		{
			if ($i === 0)
			{
				$headers[] = $line;
			}
			else
			{
				list($key, $value) = explode(': ', $line);
				$headers[$key] = $value;
			}
		}

		return $headers;
	}

	/**
	 * Gets body from cURL response.
	 *
	 * @param   string $response cURL response.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	private function getBodyFromCurlResponse($response)
	{
		$headersBodyDelimiter       = "\r\n\r\n";

		// Number of special signs in delimiter;
		$bodyStartingPositionOffset = 4;

		return substr(
			$response,
			strpos($response, $headersBodyDelimiter) + $bodyStartingPositionOffset,
			\strlen($response)
		);
	}
}
