<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Interfaces\Required\TaskQueueStorage;
use CleverReach\Infrastructure\Logger\Logger;
use CleverReach\Infrastructure\TaskExecution\Exceptions\QueueItemDeserializationException;
use CleverReach\Infrastructure\TaskExecution\Exceptions\QueueItemSaveException;
use CleverReach\Infrastructure\TaskExecution\QueueItem;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachServiceInfrastructureTaskqueuestorage implements TaskQueueStorage
{
	/**
	 * @var CleverreachRepositoryQueue
	 *
	 * @since 1.0.0
	 */
	protected $taskQueueRepository;

	/**
	 * TaskQueueStorageService constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct()
	{
		$this->taskQueueRepository = new CleverreachRepositoryQueue;
	}

	/**
	 * Creates or updates given queue item. If queue item id is not set, new queue item will be created otherwise update will be
	 * performed.
	 *
	 * @param   QueueItem $queueItem       Item to save
	 * @param   array     $additionalWhere List of key/value pairs that must be satisfied upon saving queue item. Key is queue item
	 *                                     property and value is condition value for that property.
	 *                                     Example for MySql storage:
	 *                                     $storage->save($queueItem, array('status' => 'queued')) should produce query
	 *                                     UPDATE queue_storage_table SET .... WHERE .... AND status => 'queued'
	 *
	 * @return integer Id of saved queue item
	 *
	 * @throws \CleverReach\Infrastructure\TaskExecution\Exceptions\QueueItemSaveException if queue item could not be saved
	 *
	 * @since 1.0.0
	 */
	public function save(QueueItem $queueItem, array $additionalWhere = array())
	{
		$itemId = null;

		try
		{
			$queueItemId = $queueItem->getId();

			if ($queueItemId === null || $queueItemId <= 0)
			{
				$itemId = $this->taskQueueRepository->insert($this->queueItemToArray($queueItem));
			}
			else
			{
				$this->updateQueueItem($queueItem, $additionalWhere);
				$itemId = $queueItemId;
			}
		}
		catch (\Exception $exception)
		{
			throw new QueueItemSaveException('Failed to save queue item with id: ' . $itemId, 0, $exception);
		}

		return $itemId;
	}

	/**
	 * Finds queue item by ID.
	 *
	 * @param   int $id ID of a queue item to find
	 *
	 * @return \CleverReach\Infrastructure\TaskExecution\QueueItem|null Found queue item or null when queue item does not exist
	 *
	 * @since 1.0.0
	 */
	public function find($id)
	{
		$queueItem = $this->queueItemFromArray($this->taskQueueRepository->findByPk($id));

		if ($queueItem === null)
		{
			$message = 'Failed to fetch queue item with id: ' . $id . '. Queue item  does not exist.';
			Logger::logDebug(json_encode(array('Message' => $message)));
		}

		return $queueItem;
	}

	/**
	 * Finds latest queue item by type across all queues.
	 *
	 * @param   string $type    Type of a queue item to find.
	 *
	 * @param   string $context Task context restriction if provided search will be limited to given task context. Leave empty for
	 *                          search across all task contexts.
	 *
	 * @return QueueItem|null Found queue item or null when queue item does not exist.
	 *
	 * @since 1.0.0
	 */
	public function findLatestByType($type, $context = '')
	{
		$queueItem = $this->queueItemFromArray($this->taskQueueRepository->findLatest($type));

		if ($queueItem === null)
		{
			$message = 'Failed to fetch queue item with type: ' . $type . '. Queue item  does not exist.';
			Logger::logDebug(json_encode(array('Message' => $message)));
		}

		return $queueItem;
	}

	/**
	 * Finds list of earliest queued queue items per queue. Following list of criteria for searching must be satisfied:
	 *      - Queue must be without already running queue items
	 *      - For one queue only one (oldest queued) item should be returned
	 *
	 * @param   int $limit Result set limit. By default max 10 earliest queue items will be returned
	 *
	 * @return \CleverReach\Infrastructure\TaskExecution\QueueItem[] Found queue item list
	 *
	 * @since 1.0.0
	 */
	public function findOldestQueuedItems($limit = 10)
	{
		return $this->queueItemsFromArray($this->taskQueueRepository->findOldestQueuedItems((int) $limit));
	}

	/**
	 * Finds all queue items from all queues.
	 *
	 * @param   array $filterBy List of simple search filters, where key is queue item property and value is condition
	 *                          value for that property. Leave empty for unfiltered result.
	 * @param   array $sortBy   List of sorting options where key is queue item property and value sort direction ("ASC" or "DESC").
	 *                          Leave empty for default sorting.
	 * @param   int   $start    From which record index result set should start
	 * @param   int   $limit    Max number of records that should be returned (default is 10)
	 *
	 * @return QueueItem[] Found queue item list
	 *
	 * @since 1.0.0
	 */
	public function findAll(array $filterBy = array(), array $sortBy = array(), $start = 0, $limit = 10)
	{
		return $this->queueItemsFromArray(
			$this->taskQueueRepository->findAll($filterBy, $sortBy)
		);
	}

	/**
	 * Updates database record with data from provided $queueItem.
	 *
	 * @param   QueueItem $queueItem  Queue item data.
	 * @param   array     $conditions Conditions for update.
	 *
	 * @return void
	 *
	 * @throws QueueItemDeserializationException
	 * @throws QueueItemSaveException
	 *
	 * @since 1.0.0
	 */
	private function updateQueueItem($queueItem, array $conditions = array())
	{
		// CORE can give additional conditions to preserve updating item that is possibly locked or already updated
		// this is done because of update lock.
		$conditions = array_merge($conditions, array('id' => $queueItem->getId()));

		$item = $this->taskQueueRepository->findOne($conditions);
		$this->checkIfRecordWithWhereConditionsExists($item, $conditions);

		$item = array_merge((array) $item, $this->queueItemToArray($queueItem));
		$this->taskQueueRepository->update($item, $conditions);
	}

	/**
	 * Validates if record exists.
	 *
	 * @param   array $record    Record to be checked.
	 * @param   array $condition Condition for checking.
	 *
	 * @return void
	 *
	 * @throws QueueItemSaveException
	 *
	 * @since 1.0.0
	 */
	private function checkIfRecordWithWhereConditionsExists($record, $condition)
	{
		if (empty($record))
		{
			$message = 'Failed to save queue item, update condition(s) not met.';
			Logger::logDebug(json_encode(array('Message' => $message, 'WhereCondition' => $condition)));

			throw new QueueItemSaveException($message);
		}
	}

	/**
	 * Serializes instance of QueueItem to array of values.
	 *
	 * @param   QueueItem $queueItem Item to be queued.
	 *
	 * @return array
	 * @throws QueueItemDeserializationException
	 *
	 * @since 1.0.0
	 */
	private function queueItemToArray(QueueItem $queueItem)
	{
		return array(
			'id'                    => $queueItem->getId(),
			'type'                  => $queueItem->getTaskType(),
			'status'                => $queueItem->getStatus(),
			'queueName'             => $queueItem->getQueueName(),
			'progress'              => $queueItem->getProgressBasePoints(),
			'lastExecutionProgress' => $queueItem->getLastExecutionProgressBasePoints(),
			'retries'               => $queueItem->getRetries(),
			'failureDescription'    => $queueItem->getFailureDescription(),
			'serializedTask'        => $queueItem->getSerializedTask(),
			'createTimestamp'       => $queueItem->getCreateTimestamp(),
			'queueTimestamp'        => $queueItem->getQueueTimestamp(),
			'lastUpdateTimestamp'   => $queueItem->getLastUpdateTimestamp(),
			'startTimestamp'        => $queueItem->getStartTimestamp(),
			'finishTimestamp'       => $queueItem->getFinishTimestamp(),
			'failTimestamp'         => $queueItem->getFailTimestamp(),
		);
	}

	/**
	 * Transforms array of values to QueueItem instance.
	 *
	 * @param   array $item Item to be queued from array.
	 *
	 * @return QueueItem
	 *
	 * @since 1.0.0
	 */
	private function queueItemFromArray($item)
	{
		if (empty($item))
		{
			return null;
		}

		$queueItem = new QueueItem;

		if (!empty($item->id))
		{
			$queueItem->setId((int) $item->id);
		}

		if (!empty($item->status))
		{
			$queueItem->setStatus($item->status);
		}

		if (!empty($item->queueName))
		{
			$queueItem->setQueueName($item->queueName);
		}

		if (!empty($item->progress))
		{
			$queueItem->setProgressBasePoints((int) $item->progress);
		}

		if (!empty($item->lastExecutionProgress))
		{
			$queueItem->setLastExecutionProgressBasePoints((int) $item->lastExecutionProgress);
		}

		if (!empty($item->retries))
		{
			$queueItem->setRetries((int) $item->retries);
		}

		if (!empty($item->failureDescription))
		{
			$queueItem->setFailureDescription($item->failureDescription);
		}

		if (!empty($item->serializedTask))
		{
			$queueItem->setSerializedTask($item->serializedTask);
		}

		if (!empty($item->createTimestamp))
		{
			$queueItem->setCreateTimestamp((int) $item->createTimestamp);
		}

		if (!empty($item->queueTimestamp))
		{
			$queueItem->setQueueTimestamp((int) $item->queueTimestamp);
		}

		if (!empty($item->lastUpdateTimestamp))
		{
			$queueItem->setLastUpdateTimestamp((int) $item->lastUpdateTimestamp);
		}

		if (!empty($item->startTimestamp))
		{
			$queueItem->setStartTimestamp((int) $item->startTimestamp);
		}

		if (!empty($item->finishTimestamp))
		{
			$queueItem->setFinishTimestamp((int) $item->finishTimestamp);
		}

		if (!empty($item->failTimestamp))
		{
			$queueItem->setFailTimestamp((int) $item->failTimestamp);
		}

		return $queueItem;
	}

	/**
	 * Transforms array of values to array of QueueItem instances.
	 *
	 * @param   array $items Items to be queued from array.
	 *
	 * @return QueueItem[]
	 *
	 * @since 1.0.0
	 */
	private function queueItemsFromArray($items)
	{
		$result = array();

		if (empty($items))
		{
			return $result;
		}

		foreach ($items as $item)
		{
			$result[] = $this->queueItemFromArray($item);
		}

		return $result;
	}
}
