<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\BusinessLogic\Utility\ArticleSearch\Schema\SearchableItemSchema;
use CleverReach\BusinessLogic\Utility\ArticleSearch\FilterParser;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchResult\AuthorAttribute;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchResult\BoolAttribute;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchResult\DateAttribute;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchResult\EnumAttribute;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchResult\HtmlAttribute;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchResult\ImageAttribute;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchResult\SimpleCollectionAttribute;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchResult\TextAttribute;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchResult\UrlAttribute;
use CleverReach\BusinessLogic\Utility\ArticleSearch\Validator;
use CleverReach\BusinessLogic\Utility\ArticleSearch\Exceptions\InvalidSchemaMatching;
use CleverReach\Infrastructure\Logger\Logger;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachUtilityArticleseachprovider
{
	/**
	 * Returns array of articles that satisfy provided filters.
	 *
	 * @param   string $rawFilter String representing filter in raw format.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 *
	 * @throws Exception
	 */
	public static function getSearchResults($rawFilter)
	{
		$articleRepository = new CleverreachRepositoryArticle;
		$articles = array();

		try
		{
			$schema      = self::getSearchableItemSchema();
			$contentType = $articleRepository->getContentType();
			$filters     = self::getFilters($rawFilter, $contentType, $schema);

			if (!self::verifyFilterConditions($filters))
			{
				throw new InvalidArgumentException('Wrong filter condition(s)');
			}

			$filteredArticles = $articleRepository->getFilteredArticles($filters);
			$articles         = array();

			foreach ($filteredArticles as $article)
			{
				$articles[] = array(
					'itemCode'   => $contentType,
					'id'         => $article->id,
					'title'      => $article->title,
					'date'       => new DateTime($article->created_date),
					'attributes' => self::getArticleAttributes($article),
				);
			}
		}
		catch (InvalidSchemaMatching $e)
		{
			Logger::logError($e->getMessage(), 'Integration');
		}

		return $articles;
	}

	/**
	 * Returns searchable item schema supported searchable item in the system.
	 *
	 * @return SearchableItemSchema
	 *
	 * @since 1.0.0
	 */
	public static function getSearchableItemSchema()
	{
		$articleRepository = new CleverreachRepositoryArticle;

		$contentType = $articleRepository->getContentType();
		$schema      = $articleRepository->getStaticSchema();

		return new SearchableItemSchema($contentType, $schema);
	}

	/**
	 * Filters raw filter data and returns validate`d filters for search.
	 *
	 * @param   string               $rawFilter   Raw string of filters that needs to be parsed and verified.
	 * @param   string               $contentType Searchable item type.
	 * @param   SearchableItemSchema $schema      Article schema.
	 *
	 * @return array
	 *
	 * @throws \CleverReach\BusinessLogic\Utility\ArticleSearch\Exceptions\InvalidSchemaMatching
	 *
	 * @since 1.0.0
	 */
	private static function getFilters($rawFilter, $contentType, $schema)
	{
		$filterParser = new FilterParser;
		$filters      = $filterParser->generateFilters($contentType, null, urlencode($rawFilter));

		$filterValidator = new Validator;
		$filterValidator->validateFilters($filters, $schema);

		$filterBy = array();

		foreach ($filters as $filter)
		{
			$fieldCode  = $filter->getAttributeCode();
			$fieldValue = $filter->getAttributeValue();
			$condition  = $filter->getCondition();

			if ($date = \DateTime::createFromFormat('Y-m-d\TH:i:s.u\Z', $filter->getAttributeValue()))
			{
				$fieldValue = $date->format('Y-m-d H:i:s');
			}

			$filterBy[] = array('field' => $fieldCode, 'value' => $fieldValue, 'condition' => $condition);
		}

		return $filterBy;
	}

	/**
	 * Checks if filter conditions are complying to searchable item schema.
	 *
	 * @param   array $filters Array of preformatted search filters.
	 *
	 * @return boolean
	 *
	 * @since 1.0.0
	 */
	private static function verifyFilterConditions($filters)
	{
		$articleRepository = new CleverreachRepositoryArticle;

		$staticSchema     = $articleRepository->getStaticSchema();
		$contentType      = $articleRepository->getContentType();
		$schema           = new SearchableItemSchema($contentType, $staticSchema);
		$schemaArray      = $schema->toArray();
		$schemaAttributes = $schemaArray['attributes'];

		foreach ($filters as $filter)
		{
			$schemaKey = array_search($filter['field'], array_column($schemaAttributes, 'code'), true);

			if ($schemaKey && $filter['field'] !== 'itemCode')
			{
				$schemaAttribute = $schemaAttributes[$schemaKey];

				if (!in_array($filter['condition'], $schemaAttribute['searchableExpressions'], true))
				{
					return false;
				}
			}
		}

		return true;
	}

	/**
	 * Returns all article attributes in proper format.
	 *
	 * @param   stdClass $article Article object.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 *
	 * @throws Exception
	 */
	private static function getArticleAttributes($article)
	{
		$articleRepository = new CleverreachRepositoryArticle;
		$images       = json_decode($article->images);
		$introImage   = '';
		$articleImage = '';

		$url = JUri::base() . 'index.php/' . $article->id . '-' . $article->alias;

		if (!empty($images->image_intro))
		{
			$introImage = JUri::base() . $images->image_intro;
		}

		if (!empty($images->image_fulltext))
		{
			$articleImage = JUri::base() . $images->image_fulltext;
		}

		if (!empty($article->introtext))
		{
			$content = $article->introtext . ' ' . $article->fulltext;
		}
		else
		{
			$content = $article->fulltext;
		}

		return array(
			'title'                 => new TextAttribute('title', $article->title),
			'alias'                 => new TextAttribute('alias', $article->alias),
			'status'                => new EnumAttribute('status', $articleRepository->getStatusName($article->status)),
			'content'               => new HtmlAttribute('content', $content),
			'featured'              => new BoolAttribute('featured', (bool) $article->featured),
			'category'              => new TextAttribute('category', $article->category),
			'language'              => new EnumAttribute('language', $article->language),
			'tags'                  => new SimpleCollectionAttribute('tags', $articleRepository->getArticleTags($article->id)),
			'intro_image'           => new ImageAttribute('intro_image', $introImage),
			'article_image'         => new ImageAttribute('article_image', $articleImage),
			'created_date'          => new DateAttribute('created_date', new DateTime($article->created_date)),
			'start_publishing_date' => new DateAttribute('start_publishing_date', new DateTime($article->start_publishing_date)),
			'end_publishing_date'   => new DateAttribute('end_publishing_date', new DateTime($article->end_publishing_date)),
			'author'                => new AuthorAttribute('author', $article->author),
			'author_alias'          => new TextAttribute('author_alias', $article->author_alias),
			'url'                   => new UrlAttribute('url', $url),
		);
	}
}