<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\ServiceRegister;
use CleverReach\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\Infrastructure\TaskExecution\Queue;
use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\Infrastructure\TaskExecution\QueueItem;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachUtilityHelper
{
	/**
	 * Returns URL for async process starter controller.
	 *
	 * @param   string $guid ID of the process.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public static function getAsyncProcessStartUrl($guid)
	{
		return self::getControllerUrl('site', 'asyncProcessStart', array('guid' => $guid));
	}

	/**
	 * Returns URL for callback controller.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public static function getAuthRedirectUrl()
	{
		return self::getControllerUrl('admin', 'callback.process');
	}

	/**
	 * Returns URL for refresh auth callback controller.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public static function getRefreshAuthRedirectUrl()
	{
		return self::getControllerUrl('admin', 'callback.process', array('refresh' => true));
	}

	/**
	 * Returns URL for checking status on user information.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public static function getUserInfoUrl()
	{
		return self::getControllerUrl('admin', 'checkStatus.userInfo');
	}

	/**
	 * Returns wakeup URL.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public static function getWakeupUrl()
	{
		return self::getControllerUrl('site', 'wakeUp');
	}

	/**
	 * Returns URL for initial sync.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public static function getInitialSyncUrl()
	{
		return self::getControllerUrl('admin', 'checkStatus.initialSync');
	}

	/**
	 * Returns URL for retrying synchronisation.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public static function getRetrySyncUrl()
	{
		return self::getControllerUrl('admin', 'checkStatus.retrySync');
	}

	/**
	 * Returns URL for building first e-mail.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public static function getBuildFirstEmailUrl()
	{
		return self::getControllerUrl('admin', 'checkStatus.buildFirstEmail');
	}

	/**
	 * Returns URL for configuration endpoint.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public static function getConfigurationUrl()
	{
		return self::getControllerUrl('admin', 'config.process');
	}

	/**
	 * Gets data from user and profile tables, if the user profile plugin is active.
	 *
	 * @return array
	 *
	 * @since 1.0.0
	 */
	public static function getRegisterData()
	{
		$user           = JFactory::getUser();
		$profileEnabled = JPluginHelper::isEnabled('user', 'profile');
		$fullName       = explode(' ', $user->name, 2);

		$registerData = array(
			'email'     => $user->email !== null ? $user->email : '',
			'company'   => '',
			'firstname' => isset($fullName[0]) ? $fullName[0] : '',
			'lastname'  => isset($fullName[1]) ? $fullName[1] : '',
		);

		if ($profileEnabled)
		{
			$profile = JUserHelper::getProfile($user->id)->profile;
			$street  = (!empty($profile['address1']) ? $profile['address1'] : '')
				. (!empty($profile['address2']) ? ' ' . $profile['address2'] : '');

			$registerData = array_merge(
				$registerData,
				array
				(
					'street'  => $street,
					'zip'     => isset($profile['postal_code']) ? $profile['postal_code'] : '',
					'city'    => isset($profile['city']) ? $profile['city'] : '',
					'country' => isset($profile['country']) ? $profile['country'] : '',
					'phone'   => isset($profile['phone']) ? $profile['phone'] : '',
				)
			);
		}

		return $registerData;
	}

	/**
	 * Checks if auth token exists.
	 *
	 * @return boolean
	 *
	 * @throws \InvalidArgumentException
	 *
	 * @since 1.0.0
	 */
	public static function isAuthTokenValid()
	{
		$accessToken = self::getConfigService()->getAccessToken();

		return $accessToken !== null;
	}

	/**
	 * Checks if there is an initial sync task exists in task queue. If that task is not there at all,
	 * that means that initial sync never started and it should enqueue initial sync task. If there is
	 * an initial sync task in task queue and its status is either completed or failed, that means that
	 * it is not currently in progress. In any other case it is in progress.
	 *
	 * @param   bool $enqueueTask If set to true and there is no initial sync task in the queue, the method will
	 *                            enqueue initial sync task.
	 *
	 * @return boolean
	 * @since 1.0.0
	 */
	public static function isInitialSyncInProgress($enqueueTask = false)
	{
		/** @var QueueItem $initialSyncTaskItem */
		$initialSyncTaskItem = self::getQueueService()->findLatestByType('CleverreachServiceSyncCustominitialsynctask');

		if (!$initialSyncTaskItem && $enqueueTask)
		{
			try
			{
				self::enqueueInitialSyncTask();
			}
			catch (QueueStorageUnavailableException $e)
			{
				// If task enqueue fails do nothing but report that initial sync is in progress
			}

			return true;
		}

		return $initialSyncTaskItem->getStatus() !== QueueItem::COMPLETED
			&& $initialSyncTaskItem->getStatus() !== QueueItem::FAILED;
	}

	/**
	 * Gets fail parameters for initial sync task if it failed.
	 *
	 * @return array
	 * @throws \InvalidArgumentException
	 *
	 * @since 1.0.0
	 */
	public static function getInitialSyncFailureParameters()
	{
		$params = array('isFailed' => false, 'description' => '');
		/** @var QueueItem $initialSyncTask */
		$initialSyncTask = self::getQueueService()->findLatestByType('CleverreachServiceSyncCustominitialsynctask');

		if ($initialSyncTask && $initialSyncTask->getStatus() === QueueItem::FAILED)
		{
			$params = array(
				'isFailed'    => true,
				'description' => $initialSyncTask->getFailureDescription(),
			);
		}

		return $params;
	}

	/**
	 * Checks if refresh token exists in the database.
	 *
	 * @return boolean
	 *
	 * @since 1.0.0
	 */
	public static function checkRefreshToken()
	{
		$refreshToken = self::getConfigService()->getRefreshToken();

		return !empty($refreshToken);
	}

	/**
	 * Checks if user is already configured initial sync.
	 *
	 * @return boolean
	 *   If sync configured returns true, otherwise false.
	 *
	 * @since 1.0.0
	 */
	public static function isInitialSyncConfigured()
	{
		return self::getConfigService()->isConfiguredInitialSync();
	}

	/**
	 * Checks if required core and loader classes exist in current Joomla! installation.
	 *
	 * @return boolean
	 *   If classes exist returns true, otherwise false.
	 *
	 * @since 1.0.0
	 */
	public static function requiredClassesExist()
	{
		$core   = JLibraryHelper::getLibrary('cleverreach_core', true);
		if ($core === null)
		{
			return false;
		}

		return $core->enabled
			&& JPluginHelper::isEnabled('system', 'cleverreach_loader')
			&& JPluginHelper::isEnabled('user', 'cleverreach')
			;
	}

	/**
	 * Forms controller URL.
	 *
	 * @param   string $type   Is this route an administrator or site route.
	 * @param   string $name   Controller name.
	 * @param   array  $params Query request parameters (key - value pairs)
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	private static function getControllerUrl($type, $name, array $params = array())
	{
		$queryParams = array_merge(array('option' => 'com_cleverreach', 'task' => $name), $params);

		if ($type === 'admin')
		{
			$url = JUri::root() . 'administrator/' . JRouter::getInstance('Administrator')
					->build('?' . http_build_query($queryParams))
					->toString(array('query', 'fragment'));
		}
		else
		{
			$url = JUri::root() . JRouter::getInstance('Site')
					->build('?' . http_build_query($queryParams))
					->toString(array('query', 'fragment'));
		}

		return $url;
	}

	/**
	 * Calls task queue to enqueue initial sync task.
	 *
	 * @throws QueueStorageUnavailableException
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private static function enqueueInitialSyncTask()
	{
		CleverreachUtilityTaskqueue::enqueue(new CleverreachServiceSyncCustominitialsynctask);
	}

	/**
	 * Returns configuration service.
	 *
	 * @return CleverreachServiceInfrastructureConfig
	 *
	 * @since 1.0.0
	 */
	private static function getConfigService()
	{
		return ServiceRegister::getService(Configuration::CLASS_NAME);
	}

	/**
	 * Returns queue service.
	 *
	 * @return Queue
	 *
	 * @since 1.0.0
	 */
	private static function getQueueService()
	{
		return ServiceRegister::getService(Queue::CLASS_NAME);
	}
}
