<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Interfaces\Exposed\TaskRunnerWakeup as TaskRunnerWakeupInterface;
use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\Infrastructure\Logger\Logger;
use CleverReach\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\Infrastructure\TaskExecution\Queue;
use CleverReach\Infrastructure\TaskExecution\Task;
use CleverReach\Infrastructure\TaskExecution\TaskRunnerWakeup;
use CleverReach\Infrastructure\ServiceRegister;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachUtilityTaskqueue
{
	/**
	 * Enqueues a task to the queue.
	 *
	 * @param   Task $task           Task to enqueue.
	 * @param   bool $throwException Whether this function should propagate exceptions.
	 *
	 * @return void
	 *
	 * @throws QueueStorageUnavailableException
	 *
	 * @since 1.0.0
	 */
	public static function enqueue(Task $task, $throwException = false)
	{
		try
		{
			/** @var CleverreachServiceInfrastructureConfig $configService */
			$configService = ServiceRegister::getService(Configuration::CLASS_NAME);
			$accessToken   = $configService->getAccessToken();

			if ($accessToken !== null)
			{
				/** @var Queue $queueService */
				$queueService = ServiceRegister::getService(Queue::CLASS_NAME);
				$queueService->enqueue($configService->getQueueName(), $task);
			}
		}
		catch (QueueStorageUnavailableException $ex)
		{
			Logger::logDebug(
				json_encode(
					array(
						'Message'          => 'Failed to enqueue task ' . $task->getType(),
						'ExceptionMessage' => $ex->getMessage(),
						'ExceptionTrace'   => $ex->getTraceAsString(),
						'TaskData'         => serialize($task),
					)
				),
				'Integration'
			);

			if ($throwException)
			{
				throw $ex;
			}
		}
	}

	/**
	 * Calls the wakeup on task runner.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public static function wakeup()
	{
		/** @var TaskRunnerWakeup $wakeupService */
		$wakeupService = ServiceRegister::getService(TaskRunnerWakeupInterface::CLASS_NAME);
		$wakeupService->wakeup();
	}
}
