<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\BusinessLogic\Utility\SingleSignOn\SingleSignOnProvider;
use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\Infrastructure\ServiceRegister;
use Joomla\CMS\Language\Text;
use CleverReach\BusinessLogic\Entity\Tag;

defined('_JEXEC') or die('Restricted access');

/**
 * Class CleverreachViewDashboard
 *
 * @property array  $config   Configuration values
 * @property array  $progressItems
 * @property array  $userInfo User information from configuration table
 * @property array  $failureParameters
 * @property bool   $syncFailed
 * @property string $transPart
 * @property string $prefix
 * @property bool   firstMail
 * @property array  $report
 *
 * @since 1.0.0
 */
class CleverreachViewDashboard extends JViewLegacy
{
	const CLEVERREACH_BUILD_EMAIL_URL = '/admin/mailing_create_new.php';

	/**
	 * Display dashboard view.
	 *
	 * @param   string $tpl The name of the template file to parse; automatically searches through the template paths.
	 *
	 * @return  void
	 *
	 * @since 1.0.0
	 * @throws Exception
	 */
	public function display($tpl = null)
	{
		$this->setDashboardScreenParams();
		$this->addToolBar();

		parent::display($tpl);
	}

	/**
	 * Add the page title and toolbar.
	 *
	 * @return  void
	 *
	 * @since 1.0.0
	 */
	protected function addToolBar()
	{
		JToolbarHelper::title(Text::_('COM_CLEVERREACH_TOOLBAR_TITLE'), ' icon-envelope');
	}

	/**
	 * Returns configuration service.
	 *
	 * @return CleverreachServiceInfrastructureConfig
	 *
	 * @since 1.0.0
	 */
	private function getConfigService()
	{
		return ServiceRegister::getService(Configuration::CLASS_NAME);
	}

	/**
	 * Sets parameters for dashboard screen.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 * @throws Exception
	 */
	private function setDashboardScreenParams()
	{
		$this->userInfo          = $this->getConfigService()->getUserInfo();
		$this->failureParameters = CleverreachUtilityHelper::getInitialSyncFailureParameters();

		$this->config = array(
			'buildFirstEmailUrl'            => CleverreachUtilityHelper::getBuildFirstEmailUrl(),
			'retrySyncUrl'                  => CleverreachUtilityHelper::getRetrySyncUrl(),
			'cleverReachClientId'           => $this->userInfo['id'],
			'buildEmailUrl'                 => SingleSignOnProvider::getUrl(self::CLEVERREACH_BUILD_EMAIL_URL),
			'isFirstEmailBuilt'             => $this->getConfigService()->isFirstEmailBuilt(),
			'integrationName'               => $this->getConfigService()->getIntegrationName(),
			'isInitialSyncTaskFailed'       => $this->failureParameters['isFailed'],
			'initialSyncTaskFailureMessage' => $this->failureParameters['description'],
		);

		$this->firstMail  = !$this->config['isFirstEmailBuilt'];
		$this->syncFailed = $this->config['isInitialSyncTaskFailed'];
		$this->transPart  = $this->firstMail ? 'FIRST' : 'SECOND';
		$this->prefix     = 'COM_CLEVERREACH_DASHBOARD_SCREEN_';
		$this->report = array();

		if (!$this->failureParameters['isFailed'])
		{
		    $this->report = $this->getInitialSyncReport();
		}
	}

	/**
	 * Generates initial sync report.
	 *
	 * @return array
	 *   Array with initial sync report.
	 *
	 * @throws \Exception
	 */
	private function getInitialSyncReport()
	{
		$configService = $this->getConfigService();
		$result = array(
			'isReportEnabled' => !$configService->isImportStatisticsDisplayed(),
		);

		if ($result['isReportEnabled'])
		{
			$locale = explode('-', $configService->getUserLanguage());
			$result['recipients'] = number_format(
				(int) $configService->getNumberOfSyncedRecipients(),
				0,
				Text::_('DECIMALS_SEPARATOR'),
				Text::_('THOUSANDS_SEPARATOR')
			);
			$result['name']       = $configService->getIntegrationListName();
			$result['locale']     = $locale[0];
			$result['tags']       = $this->getFormattedTags();
			$configService->setImportStatisticsDisplayed(true);
		}

		return $result;
	}

	/**
	 * Generates formatted tags.
	 *
	 * @return array
	 *   Array of formatted tags.
	 *
	 * @since 1.0.0
	 */
	private function getFormattedTags()
	{
		$result = array();
		$recipientsService = ServiceRegister::getService(\CleverReach\BusinessLogic\Interfaces\Recipients::CLASS_NAME);
		$tags = $recipientsService->getAllTags()->toArray();
		$numberOfTags = \count($tags);

		if ($numberOfTags === 0)
		{
			return $result;
		}

		$trimmedTags = array_slice($tags, 0, 3);

		/**
		 * @var int $index
		 * @var Tag $tag
		 */
		foreach ($trimmedTags as $index => $tag)
		{
			$result[] = '<div class="value" title="' . $tag->getTitle() . '">'
							. $this->getTrimmedTagName($index + 1 . ') ' . $tag->getTitle())
                      . '</div>';
		}

		if ($numberOfTags > 3)
		{
			$result[] = '<div class="value">...</div>';
		}

		return $result;
	}

	/**
	 * Trims tag name to specified length.
	 *
	 * @param   string $tag      Tag to get trimmed name.
	 * @param   int    $maxChars Max length of trimmed name.
	 * @param   string $filler   Trim filler.
	 *
	 * @return string
	 *   Trimmed tag.
	 */
	private function getTrimmedTagName($tag, $maxChars = 23, $filler = '...')
	{
		$length = \strlen($tag);
		$filterLength = \strlen($filler);

		return $length > $maxChars ? substr_replace(
			$tag,
			$filler,
			$maxChars - $filterLength,
			$length - $maxChars
		) : $tag;
	}
}
