<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\BusinessLogic\Utility\ArticleSearch\Exceptions\InvalidSchemaMatching;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchItem\SearchableItems;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchItem\SearchableItem;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchResult\SearchResult;
use CleverReach\BusinessLogic\Utility\ArticleSearch\SearchResult\SearchResultItem;
use CleverReach\Infrastructure\Logger\Logger;
use CleverReach\BusinessLogic\Utility\ArticleSearch\Validator;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachControllerArticlesearch extends CleverreachControllerBase
{
	/**
	 * @var CleverreachRepositoryArticle
	 * @since 1.0.0
	 */
	private $articleRepository;

	/**
	 * CleverreachControllerArticlesearch constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct()
	{
		parent::__construct();
		$this->articleRepository = new CleverreachRepositoryArticle;
	}

	/**
	 * Searchable items endpoint.
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function searchableItems()
	{
		if ($this->isPost())
		{
			$this->redirect404();
		}

		$searchableItems = $this->getSearchableItems();
		$this->returnJson(array('data' => $searchableItems->toArray()));
	}

	/**
	 * Searchable item schema endpoint.
	 *
	 * @param   JInput $input The application input object
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function searchableItemSchema($input)
	{
		if ($this->isPost())
		{
			$this->redirect404();
		}

		$itemCode = $input->getString('itemCode');

		if ($itemCode !== 'article')
		{
			$this->returnJson(array('data' => array()));
		}

		$searchableItemSchema = CleverreachUtilityArticleseachprovider::getSearchableItemSchema();
		$this->returnJson(array('data' => $searchableItemSchema->toArray()));
	}

	/**
	 * Performs article search with given input parameters.
	 *
	 * @param   JInput $input The application input object
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function search($input)
	{
		$itemCode = $input->getString('itemCode');

		if ($itemCode !== 'article' || $this->isPost())
		{
			$this->redirect404();
		}

		$filter = $input->getString('filter');

		if (empty($filter))
		{
			$this->returnJson(array('data' => array()));
		}

		try
		{
			$results = CleverreachUtilityArticleseachprovider::getSearchResults($input->getString('filter'));
		}
		catch (InvalidArgumentException $e)
		{
			$this->redirect404();
		}

		if (empty($results))
		{
			$this->returnJson(array('data' => array()));
		}
		else
		{
			$searchResult = new SearchResult;

			foreach ($results as $result)
			{
				$searchResult->addSearchResultItem(
					new SearchResultItem(
						$result['itemCode'],
						$result['id'],
						$result['title'],
						$result['date'],
						$result['attributes']
					)
				);
			}

			$validator = new Validator;

			try
			{
				$validator->validateSearchResults(
					$searchResult,
					CleverreachUtilityArticleseachprovider::getSearchableItemSchema()
				);
			}
			catch (InvalidSchemaMatching $e)
			{
				Logger::logError($e->getMessage(), 'Integration');
				$this->returnJson(array('error' => $e->getMessage()));
			}

			$this->returnJson(array('data' => $searchResult->toArray()));
		}
	}

	/**
	 * Returns all supported searchable items in the system.
	 *
	 * @return SearchableItems
	 *
	 * @since 1.0.0
	 */
	private function getSearchableItems()
	{
		$items           = $this->articleRepository->getSearchableItems();
		$searchableItems = new SearchableItems;

		foreach ($items as $code => $name)
		{
			$searchableItems->addSearchableItem(new SearchableItem($code, $name));
		}

		return $searchableItems;
	}
}
