<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\BusinessLogic\Interfaces\Proxy as ProxyInterface;
use CleverReach\BusinessLogic\Proxy;
use CleverReach\BusinessLogic\Sync\RecipientSyncTask;
use CleverReach\Infrastructure\Exceptions\InvalidConfigurationException;
use CleverReach\Infrastructure\ServiceRegister;
use CleverReach\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\Infrastructure\Utility\Exceptions\HttpAuthenticationException;
use CleverReach\Infrastructure\Utility\Exceptions\HttpCommunicationException;
use CleverReach\Infrastructure\Utility\Exceptions\HttpRequestException;
use CleverReach\Infrastructure\Utility\Exceptions\RefreshTokenExpiredException;
use Joomla\Input\Json;

defined('_JEXEC') or die;

class CleverreachControllerEventhandler extends CleverreachControllerBase
{
	/**
	 * Hook handle endpoint.
	 *
	 * @param   \JInput $input The application input object.
	 *
	 * @return void
	 *
	 * @throws HttpAuthenticationException
	 * @throws HttpCommunicationException
	 * @throws HttpRequestException
	 * @throws QueueStorageUnavailableException
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function handler($input)
	{
		if (!$this->isPost())
		{
			$this->confirmHandler($input);
		}
		else
		{
			$this->handleEvent($input);
		}
	}

	/**
	 * Handles request for validating webhook registration.
	 *
	 * @param  \JInput $input The application input object.
	 *
	 * @return void
	 *
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	private function confirmHandler($input)
	{
		$app      = JFactory::getApplication();
		$secret   = $this->getParam('secret', $input);
		$response = '';
		if ($secret !== null)
		{
			$verificationToken = $this->getConfigService()->getCrEventHandlerVerificationToken();
			$app->setHeader('status', 200, true);
			$response = $verificationToken . ' ' . $secret;
		}
		else
		{
			$app->setHeader('status', 400, true);
		}

		$app->sendHeaders();

		jexit($response);
	}

	/**
	 * Handles POST request.
	 *
	 * @param   \JInput $input The application input object.
	 *
	 * @return void
	 *
	 * @throws HttpAuthenticationException
	 * @throws HttpCommunicationException
	 * @throws HttpRequestException
	 * @throws QueueStorageUnavailableException
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	private function handleEvent($input)
	{
		$app         = JFactory::getApplication();
		$crCallToken = $this->getCallToken();
		if ($crCallToken === null)
		{
			$app->setHeader('status', 400, true);
		}
		elseif ($crCallToken === $this->getConfigService()->getCrEventHandlerCallToken())
		{
			/** @var Json $json */
			$json = $input->json;
			$body = json_decode($json->getRaw());
			if ($this->shouldHandleEvent($body))
			{
				$this->setSubscriptionStatus($body->payload, $body->event);
			}

			$app->setHeader('status', 200, true);
		}
		else
		{
			$app->setHeader('status', 401, true);
		}

		$app->sendHeaders();
		$app->close();
	}

	/**
	 * Method that checks whether the event should be handled by this integration.
	 *
	 * @param   stdClass $body
	 *
	 * @return boolean
	 *
	 * @since 1.0.0
	 */
	private function shouldHandleEvent($body)
	{
		if (!isset($body->payload, $body->payload->pool_id, $body->payload->group_id, $body->event))
		{
			return false;
		}

		$integrationId = $this->getConfigService()->getIntegrationId();
		$component     = JComponentHelper::getComponent('com_cleverreach', true);
		$payload       = $body->payload;

		return $component->enabled
			&& $integrationId === $payload->group_id
			&& in_array($body->event, array('receiver.subscribed', 'receiver.unsubscribed'));
	}

	/**
	 * Subscribes or unsubscribes user, depending on the request.
	 *
	 * @param   stdClass $payload
	 * @param   string   $event
	 *
	 * @return void
	 *
	 * @throws HttpAuthenticationException
	 * @throws HttpCommunicationException
	 * @throws HttpRequestException
	 * @throws QueueStorageUnavailableException
	 * @throws InvalidConfigurationException
	 * @throws RefreshTokenExpiredException
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	private function setSubscriptionStatus($payload, $event)
	{
		/** @var Proxy $proxy */
		$proxy          = ServiceRegister::getService(ProxyInterface::CLASS_NAME);
		$userRepository = new CleverreachRepositoryUser();
		$user           = $proxy->getRecipient($payload->group_id, $payload->pool_id);
		$targetUser     = $userRepository->getUserByMail($user->getEmail());
		if ($targetUser !== null)
		{
			$id = $targetUser->id;
			$userRepository->setUserNewsletterStatus($id, ($event === 'receiver.subscribed') ? 1 : 0);
			CleverreachUtilityTaskqueue::enqueue(new RecipientSyncTask(array($targetUser->id)));
		}
	}

	/**
	 * Returns CleverReach call token if it exists in request headers.
	 *
	 * @return string|null Returns call token if exists, otherwise returns false.
	 *
	 * @since 1.0.0
	 */
	private function getCallToken()
	{
		return isset($_SERVER['HTTP_X_CR_CALLTOKEN']) ? $_SERVER['HTTP_X_CR_CALLTOKEN'] : null;
	}
}